var PersInfo = function()
{
	var persinfo = {};
	var debug = 0;
	
	var patientInfo = null;
	var caseInfo = null;
	var bloodInfo = null;

	var pnrIsValid = false;
	var pnrIsAcceptable = false;
	var caseIsValid = false;

	var nofTubesIsValid = false;
	var samplingDateIsValid = true;
	var viableTumourCellsPercentIsValid = true;

	// Well validation variable for input specimen, DNA, RNA
	var wellIsValid = [];
	// Well validation variables for extracts from specimen
	var dnaWellIsValid = [];
	var rnaWellIsValid = [];

	persinfo.initPage = function()
	{
		
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', persinfo.validateStep1);
		Events.addEventHandler('caseName', 'keypress', Wizard.focusOnEnter);
		Events.addEventHandler('caseName', 'blur', persinfo.caseNameOnChange);
		Events.addEventHandler('personalNumber', 'keyup', persinfo.personalNumberOnChange);
		Events.addEventHandler('personalNumber', 'keypress', Wizard.goNextOnTabOrEnter);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', persinfo.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', persinfo.validateStep2);
		Events.addEventHandler('familyName', 'blur', persinfo.nameOnChange);
		Events.addEventHandler('allFirstNames', 'blur', persinfo.nameOnChange);
		Events.addEventHandler('allFirstNames', 'keypress', Wizard.goNextOnTabOrEnter);

		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', persinfo.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', persinfo.validateStep3);
		Events.addEventHandler('plNumber', 'blur', persinfo.plNumberOnChange);
		Events.addEventHandler('pad', 'blur', persinfo.padOnChange);
		Events.addEventHandler('nofTubes', 'keypress', Events.integerOnly);
		Events.addEventHandler('nofTubes', 'blur', persinfo.nofTubesOnChange);
		Events.addEventHandler('arrivalDate', 'blur', Wizard.validateDate);
		Events.addEventHandler('samplingDate', 'blur', persinfo.samplingDateOnChange);
		Events.addEventHandler('projectFocus', 'change', persinfo.projectFocusOnChange);
		Events.addEventHandler('site', 'change', persinfo.siteOnChange);
		Events.addEventHandler('specimenType', 'change', persinfo.specimenTypeOnChange);
		Events.addEventHandler('viableTumourCellsPercent', 'keypress', Events.integerOnly);
		Events.addEventHandler('viableTumourCellsPercent', 'blur', persinfo.viableTumourCellsPercentOnChange);
		Events.addEventHandler('otherPathNote', 'keypress', Wizard.goNextOnTab);
		Buttons.addClickHandler('selectAllMutationAnalysis', persinfo.mutationAnalysisSelectAll);
		
		// Step 4
		Events.addEventHandler('step-4', 'wizard-initialize', persinfo.initializeStep4);
		Events.addEventHandler('step-5', 'wizard-validate', persinfo.validateStep4);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('goupdate', Wizard.goRegister);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', persinfo.submit);
		
		var url = '../LabelGeneration.servlet?ID='+App.getSessionId();
		url += '&cmd=CountCases';
		Wizard.showLoadingAnimation('Loading cases...');
		Wizard.asyncJsonRequest(url, persinfo.caseInfoLoadedStep1);
	}

	persinfo.caseInfoLoadedStep1 = function(response)
	{
		var frm = document.forms['meludi'];	

		var nofCases = response.count;
		if (nofCases == null)
		{
			nofCases = 0;
		}
		var startNum = nofCases + 1;
		// Pad number with leading zeros to get 5 digits
		startNum = String('00000' + startNum).slice(-5);
		var startCase = 'ML' + startNum;
		frm.caseName.value = startCase;
		
		Doc.show('step-1');
		Doc.show('gonext');

		// Ensure that flag 'caseIsValid' is set to 'true' or error reported
		persinfo.caseNameOnChange();
	}
	
	persinfo.caseNameOnChange = function()
	{
		var frm = document.forms['meludi'];
		var caseName = frm.caseName.value.toUpperCase();
		caseIsValid = false;
		if (caseName == '')
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Missing');
			frm.caseName.focus();
			return;
		}
		if (!Meludi.isValidCaseName(caseName))
		{
			Wizard.setInputStatus('caseName', 'invalid', 'Only "ML" + 5 digits is allowed.');
			frm.caseName.focus();
			return;
		}

		frm.caseName.value = caseName;
		Wizard.setInputStatus('caseName', 'valid');
		caseIsValid = true;
		
		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetCaseInfo&caseName=' + encodeURIComponent(caseName);
		Wizard.showLoadingAnimation('Loading information about case...');
		Wizard.asyncJsonRequest(url, persinfo.onCaseInfoLoaded);
	}

	persinfo.onCaseInfoLoaded = function(response)
	{
		var frm = document.forms['meludi'];
		
		// Get biosource information from the AJAX response
		caseInfo = response.caseInfo;
		
		if (caseInfo && caseInfo.patient)
		{
			frm.personalNumber.value = caseInfo.patient.personalNumber;
			persinfo.personalNumberOnChange();
			Wizard.goNext(false);
		}
	}

	persinfo.personalNumberOnChange = function()
	{
		var frm = document.forms['meludi'];
		var pnr = frm.personalNumber.value;
		pnrIsValid = false;
		pnrIsAcceptable = true;
		
		var pnrLen = pnr.length;
		if (pnrLen != 12 && pnrLen != 10)
		{
			Wizard.setInputStatus('personalNumber', 'warning', pnrLen < 12 ? 'Too short' : 'Too long');
			return;
		}

		if (!pnr.match(/^\d+$/))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Non-standard personal number');
			return;
		}
		
		var datePart = pnr.substring(0, pnrLen - 4); // Remove last four digits
		var dateFormat = pnrLen == 10 ? 'yyMMdd' : 'yyyyMMdd';
		var isValidDate = Dates.isDate(datePart, dateFormat);
		if (!isValidDate)
		{
			var dayInMonth = parseInt(datePart.substring(-2), 10); // Last two characters
			if (dayInMonth > 60)
			{
				// 'Samordningsnummer' has day-in-month + 60; check this
				dayInMonth -= 60;
				var tmpPnr = pnr.substring(0, pnrLen - 6); // First 4 or 6 digits is year+month
				if (dayInMonth < 10) tmpPnr += '0';
				tmpPnr += dayInMonth;
				isValidDate = Dates.isDate(tmpPnr, dateFormat);
			}
		}
		if (!isValidDate)
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Not a valid date');
			return;
		}
		
		if (!Meludi.personalNumberControlDigitCheck(pnr.substr(pnrLen == 10 ? 0 : 2)))
		{
			Wizard.setInputStatus('personalNumber', 'warning', 'Invalid control digit');
			return;
		}

		Wizard.setInputStatus('personalNumber', 'valid');
		pnrIsValid = true;
		Wizard.hideGoNextConfirmation();
	}

	persinfo.validateStep1 = function(event)
	{
		if (event.detail.auto)
		{
			if (!caseIsValid || !pnrIsValid) 
			{
				if (pnrIsAcceptable)
				{
					var frm = document.forms['meludi'];
					frm.personalNumber.focus();
					Wizard.showGoNextConfirmation(true, 'Check to verify registration of non-standard personal number.');
				}
				event.preventDefault();
			}
		}
		else
		{
			if (!caseIsValid || !pnrIsAcceptable) event.preventDefault();
		}
	}

	persinfo.initializeStep2 = function(event)
	{
		var frm = document.forms['meludi'];
		var pnr = frm.personalNumber.value;
		if (pnr.length == 10 && pnrIsValid) 
		{
			var year = Meludi.fullYear(parseInt(pnr.substr(0, 2), 10));
			pnr = year + pnr.substr(2);
		}
	
		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=GetPatientInfo';
		url += '&personalNumber=' + encodeURIComponent(pnr);
		url += '&pnrIsValid=' + pnrIsValid;

		Wizard.showLoadingAnimation('Loading patient information...');
		Wizard.asyncJsonRequest(url, persinfo.patientInfoLoaded);
	}

	
	persinfo.patientInfoLoaded = function(response)
	{
		Wizard.setCurrentStep(2);
		// Get patient information from the AJAX response
		patientInfo = response.patientInfo;
		var frm = document.forms['meludi'];


		frm.patientCode.value=patientInfo.name;
		frm.familyName.value = patientInfo.familyName || '';
		frm.allFirstNames.value = patientInfo.allFirstNames || '';
		Doc.element('dateOfBirth').innerHTML = patientInfo.dateOfBirth ? Strings.encodeTags(patientInfo.dateOfBirth) : '---';
		Doc.element('gender').innerHTML = patientInfo.gender ? Strings.encodeTags(patientInfo.gender) : '---';
		frm.familyName.focus();

		if (!patientInfo.id)
		{
			if (!pnrIsValid)
			{
				Wizard.setInputStatus('gender', 'warning', 'Gender and DateOfBirth can\'t be automatically generated for non-standard Personal number');
			}
		}
		else
		{
			Doc.element('step-2-title').innerHTML = 'Existing patient: Verify names';
			frm.patientCode.disabled = true;
		}
		
		Doc.show('gonext');
		Doc.show('gocancel');
		
	}
	
	persinfo.nameOnChange = function(event)
	{
		var field = event.target;
		patientInfo[field.name] = field.value;
		if (field.value == '')
		{
			Wizard.setInputStatus(field, 'warning', 'Missing');
		}
		else
		{
			Wizard.setInputStatus(field, 'valid');
			Wizard.hideGoNextConfirmation();
		}
	}

	persinfo.validateStep2 = function(event)
	{
		var valid = true;
		var acceptable = true;
		var frm = document.forms['meludi'];

		if (frm.allFirstNames.value == '')
		{
			Wizard.setInputStatus('allFirstNames', 'warning', 'Missing');
			valid = false;
		}
		else
		{
			Wizard.setInputStatus('allFirstNames', 'valid');
		}
		
		if (frm.familyName.value == '')
		{
			Wizard.setInputStatus('familyName', 'warning', 'Missing');
			valid = false;
		}
		else
		{
			Wizard.setInputStatus('familyName', 'valid');
		}

		// New patient only
		if (!patientInfo.id)
		{
			if (frm.patientCode.value == '')
			{
				Wizard.setInputStatus('patientCode', 'invalid', 'Missing');
				frm.patientCode.focus();
				valid = false;
				acceptable = false;
			}
			else
			{
				Wizard.setInputStatus('patientCode', 'valid');
				patientInfo.name = frm.patientCode.value;
			}	
		}
		
		if (event.detail.auto)
		{
			if (!valid)
			{
				Wizard.showGoNextConfirmation(true, 'Check to verify registration with missing name(s).');
				event.preventDefault();
			}
		}
		else
		{
			if (!acceptable) event.preventDefault();
		}
		
	}
	
	persinfo.initializeStep3 = function()
	{
		// Check entered case and pnr with AJAX
		var frm = document.forms['meludi'];
		var updateMode = caseInfo.id && caseInfo.patient;
		
		// Generate list of specimen tubes
		var hasUnknownSpecimen = false;
		var hasSpecimen = caseInfo.specimen && caseInfo.specimen.length > 0;
		var hasDna = caseInfo.dna && caseInfo.dna.length > 0;
		var hasRna = caseInfo.rna && caseInfo.rna.length > 0;
		var thisCasePlNumber = null;
		var hasPlNumber = false;
		var thisCasePAD = null;
		var hasPAD = false;
		var thisCaseSamplingDate = null;
		var thisCaseExtractionDate = null;
		var thisCaseTubeContentType = caseInfo.tubeContentType;
		var thisCaseSpecimenType = null;
		var thisCaseViableTumourCellsPercent = null;

		if (hasSpecimen)
		{
			var specimenTubes = '';
			for (var i = 0; i < caseInfo.specimen.length; i++)
			{
				var specimen = caseInfo.specimen[i];
				specimenTubes += Strings.encodeTags(specimen.name);
				if (specimen.plNumber && !hasPlNumber) 
				{
					hasPlNumber = true;
					thisCasePlNumber = specimen.plNumber;
				}
				if (specimen.pad && !hasPAD) 
				{
					hasPAD = true;
					thisCasePAD = specimen.pad;
				}
				if (specimen.samplingDate && !thisCaseSamplingDate) 
				{
					thisCaseSamplingDate = specimen.samplingDate;
				}
				if (specimen.specimenType && !thisCaseSpecimenType) 
				{
					thisCaseSpecimenType = specimen.specimenType;
				}
				if (specimen.viableTumourCellsPercent && !thisCaseViableTumourCellsPercent) 
				{
					thisCaseViableTumourCellsPercent = specimen.viableTumourCellsPercent;
				}
				specimenTubes += '<br>';
			}
			Doc.element('tubeContentType').value = thisCaseTubeContentType;
			Doc.element('specimenType').value = thisCaseSpecimenType;

			Doc.show('specimenTubesSection');
			Doc.element('specimenTubes').innerHTML = specimenTubes;

			Doc.hide('nofTubesSection');
			Doc.hide('arrivalDateSection');

			Doc.show('specimenTubesSection');
			Doc.show('samplingDateSection');
			
			Doc.hide('mutationAnalysisSection');
			Doc.hide('siteSection');
			Doc.hide('sampleTypeSection');
			Doc.hide('projectFocusSection');
			Doc.hide('otherPathNoteSection');

			// Populate 'Sampling date' field
			if (thisCaseSamplingDate != null)
			{
				frm.samplingDate.value = thisCaseSamplingDate.substring(0, 8);
				persinfo.samplingDateOnChange();
			}
		
			// Populate 'Viable tumour cells percent' field
			if (thisCaseViableTumourCellsPercent != null)
			{
				frm.viableTumourCellsPercent.value = thisCaseViableTumourCellsPercent;
				persinfo.viableTumourCellsPercentOnChange();
			}
		
			frm.plNumber.value = thisCasePlNumber;
			frm.pad.value = thisCasePAD;

			if (caseInfo.id != null && typeof caseInfo.id != 'undefined')
			{
				frm.samplingDate.disabled = true;
				frm.tubeContentType.disabled = true;
				frm.specimenType.disabled = true;
				frm.viableTumourCellsPercent.disabled = true;
			}

			// Existing cases for this patient
			var hasUnknownCase = false;
			if (patientInfo.cases && patientInfo.cases.length > 0)
			{
				var cases = '';
			
				if (patientInfo.cases.length == 1 || updateMode)
				{
					if (!updateMode)
					{
						cases += '[<i>new case</i>]<br>';
					}				
				}
			
				if (updateMode)
				{
					if (caseInfo.merged)
					{
						Doc.element('step-3-title').innerHTML = 'Update case \''+Strings.encodeTags(caseInfo.originalName)+'\' (which is merged with case \'' + Strings.encodeTags(caseInfo.name) + '\')';
					}
					else
					{
						Doc.element('step-3-title').innerHTML = 'Update case \''+Strings.encodeTags(caseInfo.name)+'\'';
					}
				
				}
				else if (patientInfo.cases.length == 2)
				{
					Doc.element('step-3-title').innerHTML = 'Merge with existing case';	
				}
				else if (patientInfo.cases.length > 0)
				{
					Doc.element('step-3-title').innerHTML = 'Create new case or merge with existing case';
				}
			}

			Wizard.setCurrentStep(3);
			if (updateMode)
			{
				Doc.show('goupdate');
			}
			else
			{
				Doc.show('goregister');
			}
			Doc.show('gocancel');
			frm.plNumber.focus();
		}
		else if (hasDna)
		{
			var dnaTubes = '';
			for (var i = 0; i < caseInfo.dna.length; i++)
			{
				var dna = caseInfo.dna[i];
				dnaTubes += Strings.encodeTags(dna.name);
				if (dna.extractionDate && !thisCaseExtractionDate) 
				{
					thisCaseExtractionDate = dna.extractionDate;
				}
				dnaTubes += '<br>';
			}
			Doc.element('tubeContentType').value = thisCaseTubeContentType;

			frm.plNumber.disabled = true;
			Doc.element('plNumberMessageText').innerHTML = "No PL-number can be saved for case without specimen.";
			frm.pad.disabled = true;
			Doc.element('padMessageText').innerHTML = "No PAD/CL number can be saved for case without specimen.";

			Doc.show('specimenTubesSection');
			Doc.element('specimenTubesTitle').innerHTML = "DNA tubes";
			Doc.element('specimenTubes').innerHTML = dnaTubes;
			Doc.element('specimenTubesMessageText').innerHTML = "The DNA tube(s) associated with this case.";

			Doc.hide('nofTubesSection');
			Doc.hide('arrivalDateSection');

			Doc.show('specimenTubesSection');
			Doc.show('samplingDateSection');
			
			Doc.hide('mutationAnalysisSection');
			Doc.hide('siteSection');
			Doc.hide('sampleTypeSection');
			Doc.hide('projectFocusSection');
			Doc.hide('otherPathNoteSection');

			thisCaseSamplingDate = thisCaseExtractionDate;
			// Populate 'Sampling date' field
			if (thisCaseSamplingDate != null)
			{
				frm.samplingDate.value = thisCaseSamplingDate.substring(0, 8);
				persinfo.samplingDateOnChange();
			}
		
			frm.plNumber.value = thisCasePlNumber;
			frm.pad.value = thisCasePAD;

			if (caseInfo.id != null && typeof caseInfo.id != 'undefined')
			{
				frm.samplingDate.disabled = true;
				frm.tubeContentType.disabled = true;
				frm.specimenType.disabled = true;
				frm.viableTumourCellsPercent.disabled = true;
			}

			Wizard.setCurrentStep(3);
			if (updateMode)
			{
				Doc.show('goupdate');
			}
			else
			{
				Doc.show('goregister');
			}
			Doc.show('gocancel');
			frm.plNumber.focus();
		}
		else if (hasRna)
		{
			var rnaTubes = '';
			for (var i = 0; i < caseInfo.rna.length; i++)
			{
				var rna = caseInfo.rna[i];
				rnaTubes += Strings.encodeTags(rna.name);
				if (rna.extractionDate && !thisCaseExtractionDate) 
				{
					thisCaseExtractionDate = rna.extractionDate;
				}
				rnaTubes += '<br>';
			}
			Doc.element('tubeContentType').value = thisCaseTubeContentType;

			frm.plNumber.disabled = true;
			Doc.element('plNumberMessageText').innerHTML = "No PL-number can be saved for case without specimen.";
			frm.pad.disabled = true;
			Doc.element('padMessageText').innerHTML = "No PAD/CL number can be saved for case without specimen.";

			Doc.show('specimenTubesSection');
			Doc.element('specimenTubesTitle').innerHTML = "RNA tubes";
			Doc.element('specimenTubes').innerHTML = rnaTubes;
			Doc.element('specimenTubesMessageText').innerHTML = "The RNA tube(s) associated with this case.";

			Doc.hide('nofTubesSection');
			Doc.hide('arrivalDateSection');

			Doc.show('specimenTubesSection');
			Doc.show('samplingDateSection');
			
			Doc.hide('mutationAnalysisSection');
			Doc.hide('siteSection');
			Doc.hide('sampleTypeSection');
			Doc.hide('projectFocusSection');
			Doc.hide('otherPathNoteSection');

			thisCaseSamplingDate = thisCaseExtractionDate;
			// Populate 'Sampling date' field
			if (thisCaseSamplingDate != null)
			{
				frm.samplingDate.value = thisCaseSamplingDate.substring(0, 8);
				persinfo.samplingDateOnChange();
			}
		
			frm.plNumber.value = thisCasePlNumber;
			frm.pad.value = thisCasePAD;

			if (caseInfo.id != null && typeof caseInfo.id != 'undefined')
			{
				frm.samplingDate.disabled = true;
				frm.tubeContentType.disabled = true;
				frm.specimenType.disabled = true;
				frm.viableTumourCellsPercent.disabled = true;
			}

			Wizard.setCurrentStep(3);
			if (updateMode)
			{
				Doc.show('goupdate');
			}
			else
			{
				Doc.show('goregister');
			}
			Doc.show('gocancel');
			frm.plNumber.focus();
		}
		else
		{
			Doc.hide('specimenTubesSection');	
			Wizard.setCurrentStep(3);
			Doc.show('gocancel');
			Doc.show('gonext');
		
			frm.arrivalDate.value = Dates.formatDate(new Date(), 'yyyyMMdd');

			frm.plNumber.focus();
			frm.nofTubes.setSelectionRange(0,1);
		}
	}

	persinfo.plNumberOnChange = function()
	{
		var frm = document.forms['meludi'];
		var plNumber = frm.plNumber.value;
		if (plNumber == '')
		{
			Wizard.setInputStatus('plNumber', 'warning', 'Missing');
			return;
		}
		frm.plNumber.value = plNumber.replace(/\+/, '-');
		Wizard.setInputStatus('plNumber', 'valid');
	}

	persinfo.padOnChange = function()
	{
		var frm = document.forms['meludi'];
		var PAD = frm.pad.value;
		if (PAD == '')
		{
			Wizard.setInputStatus('pad', 'warning', 'Missing');
			return;
		}
		frm.pad.value = PAD.replace(/\+/, '-');
		Wizard.setInputStatus('pad', 'valid');
	}

	persinfo.samplingDateTimeFocus = function(event)
	{
		if (event.keyCode == 13)
		{
			var focusTo = caseInfo.specimen && caseInfo.specimen.length > 0 ? 'mutationAnalysis' : 'reasonIfNoSpecimen'
			Doc.element(focusTo).focus();
		}
	}
	
	persinfo.nofTubesOnChange = function()
	{
		nofTubesIsValid = false;

		var frm = document.forms['meludi'];
		var nofTubes = frm.nofTubes.value;
		
		if (nofTubes == '')
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Missing');
			return;
		}
		if (parseInt(nofTubes) < 1)
		{
			Wizard.setInputStatus('nofTubes', 'invalid', 'Must be at least 1');
			return;		
		}
		Wizard.setInputStatus('nofTubes', 'valid');
		nofTubesIsValid = true; 
	}

	persinfo.samplingDateOnChange = function()
	{
		var frm = document.forms['meludi'];
		samplingDateIsValid = false;
		Wizard.setInputStatus('samplingDate');
		
		var samplingDate = frm.samplingDate.value;
		
		if (samplingDate == '')
		{
			Wizard.setInputStatus('samplingDate', 'warning', 'Missing date');
		}
		else
		{
			// Auto-fill the date if it's only given with 4(MMdd) or 6(yyMMdd) digits.
			samplingDate = Meludi.autoFillDate(samplingDate, true);
			frm.samplingDate.value = samplingDate;

			if (samplingDate != '' && !Dates.isDate(samplingDate, 'yyyyMMdd'))
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Not a valid date');
				return;
			}

			var arrivalDate = frm.arrivalDate.value;
			if (arrivalDate != '' && arrivalDate < samplingDate)
			{
				Wizard.setInputStatus('samplingDate', 'invalid', 'Arrival date is before sampling date');
				return;
			}

			Wizard.setInputStatus('samplingDate', 'valid');
		}
		samplingDateIsValid = true;
	}

	persinfo.projectFocusOnChange = function()
	{
		var frm = document.forms['meludi'];
		var projectFocus = frm.projectFocus.value;

		persinfo.mutationAnalysisReset();
		
		if (projectFocus == 'melanoma')
		{
			frm.mutationAnalysisNras.checked = true;
			frm.mutationAnalysisBraf.checked = true;
			frm.mutationAnalysisKit.checked = true;
			return;
		}
		if (projectFocus == 'lungcancer')
		{
			frm.mutationAnalysisKras.checked = true;
			frm.mutationAnalysisEgfr.checked = true;
			return;
		}
		if (projectFocus == 'coloncancer')
		{
			frm.mutationAnalysisKras.checked = true;
			frm.mutationAnalysisNras.checked = true;
			frm.mutationAnalysisBraf.checked = true;
			return;
		}
		if (projectFocus == 'GIST')
		{
			frm.mutationAnalysisPdgfra.checked = true;
			frm.mutationAnalysisKit.checked = true;
			return;
		}
	}

	persinfo.mutationAnalysisReset = function()
	{
		var frm = document.forms['meludi'];

		frm.mutationAnalysisKras.checked = false;
		frm.mutationAnalysisNras.checked = false;
		frm.mutationAnalysisBraf.checked = false;
		frm.mutationAnalysisEgfr.checked = false;
		frm.mutationAnalysisAlkEml4.checked = false;
		frm.mutationAnalysisPdgfra.checked = false;
		frm.mutationAnalysisKit.checked = false;
		frm.mutationAnalysisOther.checked = false;
	}

	persinfo.mutationAnalysisSelectAll = function()
	{
		var frm = document.forms['meludi'];

		frm.mutationAnalysisKras.checked = true;
		frm.mutationAnalysisNras.checked = true;
		frm.mutationAnalysisBraf.checked = true;
		frm.mutationAnalysisEgfr.checked = true;
		frm.mutationAnalysisAlkEml4.checked = true;
		frm.mutationAnalysisPdgfra.checked = true;
		frm.mutationAnalysisKit.checked = true;
		//frm.mutationAnalysisOther.checked = true;
	}

	persinfo.viableTumourCellsPercentOnChange = function()
	{
		viableTumourCellsPercentIsValid = false;

		var frm = document.forms['meludi'];
		var viableTumourCellsPercent = frm.viableTumourCellsPercent.value;
		
		if (viableTumourCellsPercent != null)
		{
			if (parseInt(viableTumourCellsPercent) < 0)
			{
				Wizard.setInputStatus('viableTumourCellsPercent', 'invalid', 'Must be at least 0');
				return;		
			}
			if (parseInt(viableTumourCellsPercent) > 100)
			{
				Wizard.setInputStatus('viableTumourCellsPercent', 'invalid', 'Must be at most 100');
				return;		
			}
		}
		Wizard.setInputStatus('viableTumourCellsPercent', 'valid');
		viableTumourCellsPercentIsValid = true; 
	}

	persinfo.validateStep3 = function(event)
	{
		var isValid = samplingDateIsValid;
		var hasSpecimen = caseInfo.specimen && caseInfo.specimen.length > 0;
		if (hasSpecimen)
		{
			isValid &= viableTumourCellsPercentIsValid;
		}
		if (!isValid) event.preventDefault();
	}

	persinfo.initializeStep4 = function()
	{
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		var nofTubes = parseInt(frm.nofTubes.value);
		nofTubes = persinfo.fixNofTubes(tubeContentType, nofTubes);
		frm.nofTubes.value = nofTubes;
		var storageBoxSuffix = persinfo.fetchStorageBoxSuffix(tubeContentType);
		if (nofTubes > 2) Doc.addClass('step-2', 'auto-hide');
		
		var html = '';

		// Add buttons for adding standard comments for all specimens/RNA/DNA
		html += '<tr class="section-header">';
		html += '<td colspan="4">Standard comments</td>';
		html += '</tr>';

		html += '<tr class="align-top">';
		html += '<td class="subprompt"></td>';
		html += '<td><div class="button basicbutton interactable" id="addCarvedOutComment" style="font-weight: normal;">Add &quot;Utkarvad&quot; (Carved out)</div></td>';
		html += '<td class="status" id="addCarvedOutComment.status"></td>';
		html += '<td class="help"><span id="addCarvedOutComment.message" class="message"></span>Click button to add standard comment for all items.</td>';		
		html += '</tr>';

		html += '<tr class="align-top">';
		html += '<td class="subprompt"></td>';
		html += '<td><div class="button basicbutton interactable" id="addMnbComment" style="font-weight: normal;">Add &quot;MNB&quot; (Medium needle biopsy)</div></td>';
		html += '<td class="status" id="addMnbComment.status"></td>';
		html += '<td class="help"><span id="addMnbComment.message" class="message"></span>Click button to add standard comment for all items.</td>';		
		html += '</tr>';

		for (var i=0; i < nofTubes; i++)
		{
			var tubeContentTypeName = persinfo.fetchTubeContentTypeName(tubeContentType, i);
			var tubeContentTypeNameInternal = persinfo.fetchTubeContentTypeNameInternal(tubeContentType, tubeContentTypeName);
			var tubeName = persinfo.fetchTubeName(tubeContentType, caseInfo, i);
			var tubeNo = i+caseInfo.specimenFirstIndex;
			var tubeIdentifier = persinfo.fetchTubeIdentifier(tubeContentType, caseInfo, i);
			html += '<tr class="section-header">';
			html += '<td colspan="4">'+tubeContentTypeName+' Tube '+tubeName+'</td>';
			html += '</tr>';

/*
			var boxMsg = 'Box-number where the '+tubeContentTypeNameInternal+' tube is located in.';
*/
			var boxMsg = 'Box-number and position where the '+tubeContentTypeNameInternal+' tube is located.';
			var dnaBoxMsg = 'Box-number and position where the DNA extract will be located.';
			var rnaBoxMsg = 'Box-number and position where the RNA extract will be located.';
			var disabledAttribute = '';
			if (i == 0 && tubeContentType == 'Specimen')
			{
				boxMsg = 'No storage space allocated for first '+tubeContentTypeNameInternal+'; all amount expected to be used.';
				disabledAttribute = ' disabled';
			}	
			html += '<tr>';
			html += '<td class="subprompt">'+tubeContentTypeName+'&nbsp;'+tubeName+'&nbsp;&nbsp;&nbsp;'+'Box</td>';
			html += '<td class="input">';
/*
			html += '<input type="text" name="box.'+i+'" id="box.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'></td>';
*/
			html += '<input type="text" name="box.'+i+'" id="box.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'>';
			html += '&nbsp;Row<input type="text" name="row.'+i+'" id="row.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '&nbsp;Column<input type="text" name="column.'+i+'" id="column.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '</td>';
			html += '<td class="status" id="box.'+i+'.status"></td>';
			html += '<td class="help"><span id="box.'+i+'.message" class="message"></span>'+boxMsg+'<span id="position.'+i+'.message" class="message"></span></td>';
			html += '</tr>';
/*			
			html += '<tr>';
			html += '<td class="subprompt">Row</td>';
			html += '<td class="input">';
			html += '<input type="text" name="row.'+i+'" id="row.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '&nbsp;Column<input type="text" name="column.'+i+'" id="column.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
			html += '</td>';
			html += '<td class="status" id="position.'+i+'.status"></td>';
			html += '<td class="help"><span id="position.'+i+'.message" class="message"></span></td>';		
			html += '</tr>';
*/

			if (tubeContentType == 'Specimen')
			{
				disabledAttribute = '';
				html += '<tr>';
				html += '<td class="subprompt">DNA extract'+'&nbsp;'+tubeName+'.d&nbsp;&nbsp;&nbsp;'+'Box</td>';
				html += '<td class="input">';
				html += '<input type="text" name="dnaBox.'+i+'" id="dnaBox.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'>';
				html += '&nbsp;Row<input type="text" name="dnaRow.'+i+'" id="dnaRow.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '&nbsp;Column<input type="text" name="dnaColumn.'+i+'" id="dnaColumn.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '</td>';
				html += '<td class="status" id="dnaBox.'+i+'.status"></td>';
				html += '<td class="help"><span id="dnaBox.'+i+'.message" class="message"></span>'+dnaBoxMsg+'<span id="dnaPosition.'+i+'.message" class="message"></span></td>';
				html += '</tr>';

				html += '<tr>';
				html += '<td class="subprompt">RNA extract'+'&nbsp;'+tubeName+'.r&nbsp;&nbsp;&nbsp;'+'Box</td>';
				html += '<td class="input">';
				html += '<input type="text" name="rnaBox.'+i+'" id="rnaBox.'+i+'" style="width: 12em;" maxlength="12"'+disabledAttribute+'>';
				html += '&nbsp;Row<input type="text" name="rnaRow.'+i+'" id="rnaRow.'+i+'" style="text-transform:uppercase; width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '&nbsp;Column<input type="text" name="rnaColumn.'+i+'" id="rnaColumn.'+i+'" style="width: 3em;" maxlength="2"'+disabledAttribute+'>';
				html += '</td>';
				html += '<td class="status" id="rnaBox.'+i+'.status"></td>';
				html += '<td class="help"><span id="rnaBox.'+i+'.message" class="message"></span>'+rnaBoxMsg+'<span id="rnaPosition.'+i+'.message" class="message"></span></td>';
				html += '</tr>';

				html += '<tr>';
				html += '<td class="subprompt">Number of sections</td>';
				html += '<td class="input">';
				html += '<input type="text" name="nofSections.'+i+'" id="nofSections.'+i+'" style="width: 3em;" maxlength="2" value="1">';
				html += '</td>';
				html += '<td class="status" id="nofSections.'+i+'.status"></td>';
				html += '<td class="help"><span id="nofSections.'+i+'.message" class="message"></span>Number of sections (slices) in FFPE block.</td>';		
				html += '</tr>';
			}

			html += '<tr class="align-top">';
			html += '<td class="subprompt">Operator delivery comment</td>';
			html += '<td class="input"><textarea rows="3" name="comment.'+i+'" id="comment.'+i+'"></textarea></td>';
			html += '<td class="status" id="comment.'+i+'.status"></td>';
			html += '<td class="help"><span id="comment.'+i+'.message" class="message"></span>In this field, enter information specific for '+tubeContentTypeNameInternal+' tube ' +tubeIdentifier+'.</td>';		
			html += '</tr>';
		}
		Doc.element('specimen-tubes').innerHTML = html;

		// Add click handlers for buttons for adding standard comments for all specimens/RNA/DNA
		var addCarvedOutCommentGeneralButton = Doc.element('addCarvedOutComment');
		Buttons.addClickHandler(addCarvedOutCommentGeneralButton, persinfo.addCommentGeneral, {'nofTubes': nofTubes, 'text-field': 'comment.', 'text': 'Utkarvad.'});
		var addMnbCommentGeneralButton = Doc.element('addMnbComment');
		Buttons.addClickHandler(addMnbCommentGeneralButton, persinfo.addCommentGeneral, {'nofTubes': nofTubes, 'text-field': 'comment.', 'text': 'MNB.'});

		Wizard.setCurrentStep(4);
		Doc.show('goregister');
		Doc.show('gocancel');
		frm['comment.0'].focus();
		
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=FindStoragePositions';	
		url += '&nofTubes='+frm.nofTubes.value;
		url += '&storageBoxSuffix='+storageBoxSuffix;
		
		Wizard.showLoadingAnimation('Looking for empty box positions...');
		Wizard.asyncJsonRequest(url, persinfo.storagePositionsLoaded);
	}
	
	persinfo.addCommentGeneral = function(event)
	{
		var nofTubes = Data.get(event.currentTarget, 'nofTubes', event.currentTarget.id);
		var textFieldBase = Data.get(event.currentTarget, 'text-field', event.currentTarget.id);
		var text = Data.get(event.currentTarget, 'text', event.currentTarget.id);
		for (var i=0; i < nofTubes; i++)
		{
			var textField = Doc.element(textFieldBase+i);
			if (textField)
			{
				if (text && text != '')
				{
					if (textField.value != '')
					{
						textField.value += ' ';
					}
					textField.value += text;
				}
			}
		}
	}
	
	persinfo.addComment = function(event)
	{
		var textField = Doc.element(Data.get(event.currentTarget, 'text-field', event.currentTarget.id));
		var text = Data.get(event.currentTarget, 'text', event.currentTarget.id);
		if (textField)
		{
			if (text && text != '')
			{
				if (textField.value != '')
				{
					textField.value += ' ';
				}
				textField.value += text;
			}
		}
	}
	
	persinfo.storagePositionsLoaded = function(response)
	{
		var freeWells = response.wells;
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		var nofTubes = parseInt(frm.nofTubes.value);

		// No storage space is allocated for first specimen,
		// since all amount is expected to be used for analysis.
		// Specimen tube with i == 0 is therefore skipped.
		if (tubeContentType == 'Specimen')
		{
			for (var i = 0; i < nofTubes; i++)
			{
				wellIndex = 3*i;
				var specWell = null;
				if (i > 0)
				{
					specWell = freeWells[wellIndex];
					if (!specWell || specWell == '') 
					{
						Wizard.setInputStatus('box.'+i, 'invalid', 'Missing box');
						return;
					}
				}
				var dnaWell = freeWells[wellIndex+1];
				if (!dnaWell || dnaWell == '') 
				{
					Wizard.setInputStatus('dnaBox.'+i, 'invalid', 'Missing box');
					return;
				}
				var rnaWell = freeWells[wellIndex+2];
				if (!rnaWell || rnaWell == '') 
				{
					Wizard.setInputStatus('rnaBox.'+i, 'invalid', 'Missing box');
					return;
				}

				// Update form fields for storage plates
				if (specWell)
				{
					frm['box.'+i].value = specWell.bioPlate.name;
					frm['row.'+i].value = Meludi.wellToAlpha(specWell.row);
					frm['column.'+i].value = specWell.column+1;
				}
			
				if (dnaWell)
				{
					frm['dnaBox.'+i].value = dnaWell.bioPlate.name;
					frm['dnaRow.'+i].value = Meludi.wellToAlpha(dnaWell.row);
					frm['dnaColumn.'+i].value = dnaWell.column+1;
				}

				if (rnaWell)
				{
					frm['rnaBox.'+i].value = rnaWell.bioPlate.name;
					frm['rnaRow.'+i].value = Meludi.wellToAlpha(rnaWell.row);
					frm['rnaColumn.'+i].value = rnaWell.column+1;
				}

				// Move focus to next field
				Events.addEventHandler('box.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'row.'+i });
				Events.addEventHandler('row.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'column.'+i });
				Events.addEventHandler('column.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'dnaBox.'+i });

				Events.addEventHandler('dnaBox.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'dnaRow.'+i });
				Events.addEventHandler('dnaRow.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'dnaColumn.'+i });
				Events.addEventHandler('dnaColumn.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'rnaBox.'+i });

				Events.addEventHandler('rnaBox.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'rnaRow.'+i });
				Events.addEventHandler('rnaRow.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'rnaColumn.'+i });
				Events.addEventHandler('rnaColumn.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'nofSections.'+i });

				Events.addEventHandler('nofSections.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'comment.'+i });
			
				// Form validation
				Events.addEventHandler('column.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('box.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'Specimen', 'index': i });
				Events.addEventHandler('row.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'Specimen', 'index': i });
				Events.addEventHandler('column.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'Specimen', 'index': i });
				Events.addEventHandler('box.'+i, 'keyup', persinfo.boxOnKeyUp, { 'boxtype': 'Specimen'});

				Events.addEventHandler('dnaColumn.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('dnaBox.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'DNA', 'index': i });
				Events.addEventHandler('dnaRow.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'DNA', 'index': i });
				Events.addEventHandler('dnaColumn.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'DNA', 'index': i });
				Events.addEventHandler('dnaBox.'+i, 'keyup', persinfo.boxOnKeyUp, { 'boxtype': 'DNA'});

				Events.addEventHandler('rnaColumn.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('rnaBox.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'RNA', 'index': i });
				Events.addEventHandler('rnaRow.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'RNA', 'index': i });
				Events.addEventHandler('rnaColumn.'+i, 'change', persinfo.wellOnChange, { 'boxtype': 'RNA', 'index': i });
				Events.addEventHandler('rnaBox.'+i, 'keyup', persinfo.boxOnKeyUp, { 'boxtype': 'RNA'});

				Events.addEventHandler('nofSections.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('nofSections.'+i, 'change', persinfo.nofSectionsOnChange, { 'index': i });
			
				wellIsValid[i] = true;
				dnaWellIsValid[i] = true;
				rnaWellIsValid[i] = true;
/*
				Wizard.setInputStatus('position.'+i, 'valid');
*/
				Wizard.setInputStatus('box.'+i, 'valid');
				Wizard.setInputStatus('dnaBox.'+i, 'valid');
				Wizard.setInputStatus('rnaBox.'+i, 'valid');
				if (tubeContentType == 'Specimen')
				{
					Wizard.setInputStatus('nofSections.'+i, 'valid');
				}
			}
		}
		else if (tubeContentType == 'DNA' || tubeContentType == 'RNA' || tubeContentType == 'DNA_RNA')
		{
			for (var i = 0; i < nofTubes; i++)
			{
				var wellIndex = i;
				var well = freeWells[wellIndex];
				if (!well || well == '') 
				{
					Wizard.setInputStatus('box.'+i, 'invalid', 'Missing box');
					return;
				}
				frm['box.'+i].value = well.bioPlate.name;
				frm['row.'+i].value = Meludi.wellToAlpha(well.row);
				frm['column.'+i].value = well.column+1;
			
				// Move focus to next field
				Events.addEventHandler('box.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'row.'+i });
				Events.addEventHandler('row.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'column.'+i });
				Events.addEventHandler('column.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'nofSections.'+i });
				Events.addEventHandler('nofSections.'+i, 'keypress', Wizard.focusOnEnter, { 'next-focus': 'comment.'+i });
			
				// Form validation
				Events.addEventHandler('column.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('box.'+i, 'change', persinfo.wellOnChange, { 'index': i });
				Events.addEventHandler('row.'+i, 'change', persinfo.wellOnChange, { 'index': i });
				Events.addEventHandler('column.'+i, 'change', persinfo.wellOnChange, { 'index': i });
				Events.addEventHandler('box.'+i, 'keyup', persinfo.boxOnKeyUp);
				Events.addEventHandler('nofSections.'+i, 'keypress', Events.integerOnly);
				Events.addEventHandler('nofSections.'+i, 'change', persinfo.nofSectionsOnChange, { 'index': i });
			
				wellIsValid[i] = true;
/*
				Wizard.setInputStatus('position.'+i, 'valid');
*/
				Wizard.setInputStatus('box.'+i, 'valid');
				if (tubeContentType == 'Specimen')
				{
					Wizard.setInputStatus('nofSections.'+i, 'valid');
				}
			}
		}
	}

	persinfo.fixNofTubes = function(tubeContentType, nofTubes)
	{
		if (tubeContentType == 'DNA')
		{
			nofTubes = 1;
		}
		else if (tubeContentType == 'RNA')
		{
			nofTubes = 1;
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			nofTubes = 2;
		}
		return nofTubes;
	}
	
	persinfo.fetchStorageBoxSuffix = function(tubeContentType)
	{
		var storageBoxSuffix = null;
		if (tubeContentType == 'Specimen')
		{
			storageBoxSuffix = '_sp';
		}
		else if (tubeContentType == 'DNA')
		{
			storageBoxSuffix = '_dna';
		}
		else if (tubeContentType == 'RNA')
		{
			storageBoxSuffix = '_rna';
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			storageBoxSuffix = '_dna_rna';
		}
		return storageBoxSuffix;
	}
	
	persinfo.fetchTubeContentTypeName = function(tubeContentType, tubeIndex)
	{
		var tubeContentTypeName = tubeContentType;
		if (tubeContentType == 'DNA_RNA')
		{
			if (tubeIndex == 0)
			{
				tubeContentTypeName = 'DNA';
			}
			else if (tubeIndex == 1)
			{
				tubeContentTypeName = 'RNA';
			}
		}
		return tubeContentTypeName;
	}
	
	persinfo.fetchTubeContentTypeNameInternal = function(tubeContentType, tubeContentTypeName)
	{
		var tubeContentTypeNameInternal = tubeContentTypeName;
		// Convert all names except abbreviations to all lower case
		if (tubeContentType == 'Specimen')
		{
			tubeContentTypeNameInternal = tubeContentTypeName.toLowerCase();
		}
		return tubeContentTypeNameInternal;
	}
	
	persinfo.fetchTubeName = function(tubeContentType, caseInfo, tubeIndex)
	{
		var tubeName = Strings.encodeTags(caseInfo.originalName);
		if (tubeContentType == 'Specimen')
		{
			var tubeNo = tubeIndex + caseInfo.specimenFirstIndex;
			tubeName = tubeName+'.'+tubeNo;
		}
		else if (tubeContentType == 'DNA')
		{
			tubeName = tubeName+'.d';
		}
		else if (tubeContentType == 'RNA')
		{
			tubeName = tubeName+'.r';
		}
		else if (tubeContentType == 'DNA_RNA')
		{
			if (tubeIndex == 0)
			{
				tubeName = tubeName+'.d';
			}
			else if (tubeIndex == 1)
			{
				tubeName = tubeName+'.r';
			}
		}
		return tubeName;
	}
	
	persinfo.fetchTubeIdentifier = function(tubeContentType, tubeName, tubeNo)
	{
		var tubeIdentifier = tubeName;
		if (tubeContentType == 'Specimen')
		{
			tubeIdentifier = tubeNo;
		}
		return tubeIdentifier;
	}
	
	persinfo.boxOnKeyUp = function(event)
	{
		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		
		var boxType = Data.get(event.currentTarget, 'boxtype');
		var index = Data.int(event.currentTarget, 'index');

		// Get box prefix, default is for input tubes
		var boxPrefix = 'box.';
		if (boxType && boxType == 'DNA')
		{
			boxPrefix = 'dnaBox.';
		}
		if (boxType && boxType == 'RNA')
		{
			boxPrefix = 'rnaBox.';
		}

		// Check box name
		var boxIsValid = true;
		var box = frm[boxPrefix+index].value;
		Wizard.setInputStatus(boxPrefix+index);
		// Check for prefix "ML_"
		if (box.indexOf('ML_') != 0)
		{
			boxIsValid = false;
		}
		var suffix = '_sp';
		if (tubeContentType == 'DNA' || (boxType && boxType == 'DNA'))
		{
			suffix = '_dna';
		}
		else if (tubeContentType == 'RNA' || (boxType && boxType == 'RNA'))
		{
			suffix = '_rna';
		}
		// Check for suffix
		if (box.indexOf(suffix) != (box.length - suffix.length))
		{
			boxIsValid = false;
		}
		
		if (!boxIsValid)
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Box name must start with "ML_" and end with "'+suffix+'"');
			return;
		}
	}
	
	persinfo.wellOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		var target = event.currentTarget;

		var boxType = Data.get(target, 'boxtype');
		var index = Data.int(target, 'index');

		// Get well position prefixes, default is for input tubes
		var boxPrefix = 'box.';
		var rowPrefix = 'row.';
		var columnPrefix = 'column.';
		if (boxType && boxType == 'DNA')
		{
			boxPrefix = 'dnaBox.';
			rowPrefix = 'dnaRow.';
			columnPrefix = 'dnaColumn.';
		}
		if (boxType && boxType == 'RNA')
		{
			boxPrefix = 'rnaBox.';
			rowPrefix = 'rnaRow.';
			columnPrefix = 'rnaColumn.';
		}
		var box = frm[boxPrefix+index].value;
		var row = frm[rowPrefix+index].value;
		var column = frm[columnPrefix+index].value;
		
		//wellIsValid[index] = true;
		if (!boxType || boxType == 'Specimen')
		{
			wellIsValid[index] = false;
		}
		else if (boxType && boxType == 'DNA')
		{
			dnaWellIsValid[index] = false;
		}
		else if (boxType && boxType == 'RNA')
		{
			rnaWellIsValid[index] = false;
		}
		else
		{
			wellIsValid[index] = false;
		}
		Wizard.setInputStatus(boxPrefix+index);
/*
		Wizard.setInputStatus('position.'+index);
*/
		
		if (box == '') 
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Missing box');
			return;
		}
		
		// Position errors are reported on box message line		
		if (row == '')
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Missing row');
			return;
		}

		if (column == '')
		{
			Wizard.setInputStatus(boxPrefix+index, 'invalid', 'Missing column');
			return;
		}
		
		// Check for duplicates
		var nofTubes = parseInt(frm.nofTubes.value);
		for (var i=0; i < nofTubes; i++)
		{		
			if (i != index)
			{
				if (box == frm.elements[boxPrefix+i].value && row == frm[rowPrefix+i].value && column == frm[columnPrefix+i].value)
				{
					Wizard.setInputStatus(boxPrefix+index, 'invalid', 'The position is already used once in this wizard');				
					return;
				}
			}
		}

		//Wizard.setInputStatus('position.'+index, 'checking', 'Checking...');
		Wizard.setInputStatus(boxPrefix+index, 'checking', 'Checking...');
		
		// Seems to be ok -- check if used by another sample
		var request = Ajax.getXmlHttpRequest();
		var url = '../SpecimenTubeRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=ValidateWell';
		url += '&box=' + encodeURIComponent(box);	
		url += '&row=' + encodeURIComponent(row);
		url += '&column=' + encodeURIComponent(column);
		
		Wizard.asyncJsonRequest(url, function(response) { persinfo.onBoxValidated(response, boxType, index) } );
	}
	
	persinfo.onBoxValidated = function(response, boxType, index)
	{
		// Get well position prefixes, default is for input tubes
		var boxPrefix = 'box.';
		if (boxType && boxType == 'DNA')
		{
			boxPrefix = 'dnaBox.';
		}
		if (boxType && boxType == 'RNA')
		{
			boxPrefix = 'rnaBox.';
		}

		//Wizard.setInputStatus('position.'+index);
		Wizard.setInputStatus(boxPrefix+index);
		if (response.message)
		{
			//Wizard.setInputStatus(response.box ? 'box.'+index : 'position.'+index, 'invalid', Strings.encodeTags(response.message));
			Wizard.setInputStatus(boxPrefix+index, 'invalid', Strings.encodeTags(response.message));
			return;
		}
		
		//wellIsValid[index] = true;
		if (!boxType || boxType == 'Specimen')
		{
			wellIsValid[index] = true;
		}
		else if (boxType && boxType == 'DNA')
		{
			dnaWellIsValid[index] = true;
		}
		else if (boxType && boxType == 'RNA')
		{
			rnaWellIsValid[index] = true;
		}
		else
		{
			wellIsValid[index] = true;
		}
		Wizard.setInputStatus(boxPrefix+index, 'valid');
/*
		Wizard.setInputStatus('position.'+index, 'valid');
*/
	}

	persinfo.nofSectionsOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		var target = event.currentTarget;

		var index = Data.int(target, 'index');
		var nofSections = frm['nofSections.'+index].value;

		Wizard.setInputStatus('nofSections.'+index);

		if (nofSections == '')
		{
			Wizard.setInputStatus('nofSections'+index, 'invalid', 'Missing');
			return;
		}
		if (parseInt(nofSections) < 1)
		{
			Wizard.setInputStatus('nofSections'+index, 'invalid', 'Must be at least 1');
			return;		
		}
		Wizard.setInputStatus('nofSections'+index, 'valid');
	}

	persinfo.validateStep4 = function(event)
	{
		var valid = false;

		var frm = document.forms['meludi'];
		var tubeContentType = frm.tubeContentType.value;
		if (tubeContentType == 'Specimen')
		{
			for (var i = 0; i < wellIsValid.length; i++)
			{
				// No storage space is allocated for first specimen,
				// since all amount is expected to be used for analysis.
				// Specimen tube with i == 0 is therefore skipped.
				if (i > 0)
				{
					if (!wellIsValid[i])
					{
						event.preventDefault();
						return;
					}
				}
				if (!dnaWellIsValid[i])
				{
					event.preventDefault();
					return;
				}
				if (!rnaWellIsValid[i])
				{
					event.preventDefault();
					return;
				}
			}
		}
		else
		{
			// Input DNA, RNA, or DNA + RNA
			for (var i = 0; i < wellIsValid.length; i++)
			{
				if (!wellIsValid[i])
				{
					event.preventDefault();
					return;
				}
			}
		}
	}

	persinfo.submit = function()
	{
		var frm = document.forms['meludi'];
		var updateMode = caseInfo.id;

		caseInfo.plNumber = frm.plNumber.value;
		caseInfo.pad = frm.pad.value;
		caseInfo.samplingDate = Strings.trim(frm.samplingDate.value);
		caseInfo.tubeContentType = frm.tubeContentType.value;
		if (!updateMode)
		{
			caseInfo.arrivalDate = Strings.trim(frm.arrivalDate.value);
			caseInfo.mutationAnalysisKras = frm.mutationAnalysisKras.checked;
			caseInfo.mutationAnalysisNras = frm.mutationAnalysisNras.checked;
			caseInfo.mutationAnalysisBraf = frm.mutationAnalysisBraf.checked;
			caseInfo.mutationAnalysisEgfr = frm.mutationAnalysisEgfr.checked;
			caseInfo.mutationAnalysisAlkEml4 = frm.mutationAnalysisAlkEml4.checked;
			caseInfo.mutationAnalysisPdgfra = frm.mutationAnalysisPdgfra.checked;
			caseInfo.mutationAnalysisKit = frm.mutationAnalysisKit.checked;
			caseInfo.mutationAnalysisOther = frm.mutationAnalysisOther.checked;
			caseInfo.site = frm.site.value;
			caseInfo.specimenType = frm.specimenType.value;
			caseInfo.projectFocus = frm.projectFocus.value;
			caseInfo.viableTumourCellsPercent = frm.viableTumourCellsPercent.value;
			caseInfo.otherPathNote = frm.otherPathNote.value;
		
			var nofTubes = parseInt(frm.nofTubes.value);
			caseInfo.specimen = [];
			for (var i = 0; i < nofTubes; i++)
			{
				var specimenTube ={};
				specimenTube.name = persinfo.fetchTubeName(caseInfo.tubeContentType, caseInfo, i);

				specimenTube.box = frm['box.'+i].value;
				specimenTube.row = frm['row.'+i].value;
				specimenTube.column = frm['column.'+i].value;
				var nofSections = null;
				if (caseInfo.tubeContentType == 'Specimen')
				{
					// Storage position for DNA extract from specimen
					specimenTube.dnaBox = frm['dnaBox.'+i].value;
					specimenTube.dnaRow = frm['dnaRow.'+i].value;
					specimenTube.dnaColumn = frm['dnaColumn.'+i].value;

					// Storage position for RNA extract from specimen				
					specimenTube.rnaBox = frm['rnaBox.'+i].value;
					specimenTube.rnaRow = frm['rnaRow.'+i].value;
					specimenTube.rnaColumn = frm['rnaColumn.'+i].value;
				
					nofSections = frm['nofSections.'+i].value;
				}
				specimenTube.nofSections = nofSections;
				specimenTube.comment = frm['comment.'+i].value;
				specimenTube.specimenType = frm.specimenType.value;
				caseInfo.specimen[i] = specimenTube;
			}
		}
		var submitInfo = {};
		submitInfo.patientInfo = patientInfo;
		submitInfo.caseInfo = caseInfo;

		var url = '../PersonalRegistration.servlet?ID='+App.getSessionId();
		url += '&cmd=' + (updateMode ? 'UpdateCase' : 'CreateCase');
		Wizard.showLoadingAnimation('Registering case...');
		Wizard.asyncJsonRequest(url, persinfo.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	persinfo.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return persinfo;
}();

Doc.onLoad(PersInfo.initPage);

