var ExtractionReg = function()
{
	var exreg = {};
	var debug = 0;
	
	exreg.REAGENTS = [
		'xyleneSpecimen', 'etoh995Specimen', 'bufferPkdSpecimen', 'bufferAtlSpecimen',
		'proteinaseKSpecimen', 'rnaseASpecimen', 'allPrepFfpeKit', 'etoh995',
		'bufferRlt', 'bufferFrn', 'bufferRpe', 'bufferAl',
		'bufferAw1', 'bufferAw2', 'bufferAte', 'bufferRdd',
		'rnaseFreeWater', 'dnaseMix', 'rneasyMinEluteSpinColumn', 'qiaampMinEluteSpinColumn'
		];

	var DEFAULT_RNA_VOLUME = 25;
	var DEFAULT_DNA_VOLUME = 25;
	var MINIMAL_PRE_NORMALIZE_QUANTITY = 1.1; // µg
	var EXTRA_QIACUBE_SAMPLE_NAME_START = 'E';
	var QUBIT_DILUTION_FACTOR_DEFAULT = 200.0;
	var protocolMap = {};

	var extraItemsList = [];
	var qiacubeSortedPrefixes = 'CABDEFGHIJKLMNOPQRSTUVWXYZ';
	var qiacubeDateIsAfterIsolationDate = true;
	var qiacubeRunNoIsValid = false;
	var qiacubePositionsAreValid = false;
	var reagentFileIsValid = false;
	var nanoDropFileIsValid = false;
	var qubitFileIsValid = false;
	var qPcrFileIsValid = false;
	var counterBalanceInfoRow = false;
	var trackingSheetPreview = true;
	var createdRna = [];
	
	// Page initialization
	exreg.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', exreg.validateStep1);
		//Events.addEventHandler('extractSourceItems', 'change', exreg.sourceItemsOnChange);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', exreg.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', exreg.validateStep2);
		
		Events.addEventHandler('isolationDate', 'change', Wizard.validateDate);
		Events.addEventHandler('qiacubeDate', 'change', Wizard.validateDate);
		Events.addEventHandler('isolationDate', 'change', exreg.qiacubeDateOnChange);
		Events.addEventHandler('qiacubeDate', 'change', exreg.qiacubeDateOnChange);
		Events.addEventHandler('qiacubeRunNo', 'change', exreg.qiacubeRunNoOnChange);
		Events.addEventHandler('reagentFile', 'change', exreg.reagentFileOnChange);
		
		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', exreg.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', exreg.validateStep3);

		Events.addEventHandler('nanoDropFile', 'change', exreg.nanoDropFileOnChange);
		Events.addEventHandler('qubitFile', 'change', exreg.qubitFileOnChange);
		Events.addEventHandler('qubitFileDilutionFactor', 'change', exreg.qubitFileDilutionFactorOnChange);
		Events.addEventHandler('qPcrFile', 'change', exreg.qPcrFileOnChange);

		Buttons.addClickHandler('btnFakeNanoDropFile', exreg.generateFakeNanoDropFile);		
		Buttons.addClickHandler('btnFakeDnaInput', exreg.generateFakeDnaInput);
		Buttons.addClickHandler('btnFakeQPcrFile', exreg.generateFakeQPcrFile);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('gocreate', exreg.createProtocol);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', exreg.submit);

/*
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnprocessedItems';		
		Wizard.showLoadingAnimation('Loading unprocessed items...');
		Wizard.asyncJsonRequest(url, exreg.initializeStep1);
*/
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnprocessedStartPlates';		
		Wizard.showLoadingAnimation('Loading unprocessed items...');
		Wizard.asyncJsonRequest(url, exreg.initializeStep1);
	}
	
	exreg.initializeStep1 = function(response)
	{
		var frm = document.forms['meludi'];
		var plates = response.plates;
		
		var startPlateList = frm.startPlate;
		if (plates.length > 0)
		{
			for (var i=0; i < plates.length; i++)
			{
				var plate = plates[i];
				var name = (i+1) + ': ' + Strings.encodeTags(plate.name);
				if (plate.sourceItemNames)
				{
					name += ' -- (';
					name += plate.sourceItemNames.length;
					if (plate.numberExtraQiacubeItems)
					{
						name += ' + ' + plate.numberExtraQiacubeItems;
					}
					name += ')';
				}
				var selected = i == (plates.length - 1);
				var option = new Option(name, plate.id, selected, selected);
				option.plate = plate;
				startPlateList.options[startPlateList.length] = option;
			}
			//exreg.sourceItemsOnChange();
		}
		else
		{
			Wizard.setFatalError('No plates available for processing.');
			return;
		}
/*
		var specimens = response.specimens;
		var rnas = response.rnas;
		var dnas = response.dnas;
		
		var sourceItemList = frm.extractSourceItems;
		if (specimens.length > 0 || rnas.length > 0 || dnas.length > 0)
		{
			var numItems = specimens.length;
			if (specimens.length > 0)
			{
				for (var i=0; i < specimens.length; i++)
				{
					var specimen = specimens[i];
					var name = (i+1) + ': ' + Strings.encodeTags(specimen.name);
					if (specimen.bioWell)
					{
						name += ' -- ' + Strings.encodeTags(specimen.bioWell.bioPlate.name + ' (' + specimen.bioWell.location+')');
					}
					var selected = i < 24;
					var option = new Option(name, specimen.id, selected, selected);
					option.specimen = specimen;
					sourceItemList.options[sourceItemList.length] = option;
				}
			}

			if (rnas.length > 0)
			{
				for (var i=0; i < rnas.length; i++)
				{
					var rna = rnas[i];
					var name = (numItems+i+1) + ': ' + Strings.encodeTags(rna.name);
					if (rna.bioWell)
					{
						name += ' -- ' + Strings.encodeTags(rna.bioWell.bioPlate.name + ' (' + rna.bioWell.location+')');
					}
					var selected = i < 24-numItems;
					var option = new Option(name, rna.id, selected, selected);
					option.rna = rna;
					sourceItemList.options[sourceItemList.length] = option;
				}
			}
			numItems += rnas.length;

			if (dnas.length > 0)
			{
				for (var i=0; i < dnas.length; i++)
				{
					var dna = dnas[i];
					var name = (numItems+i+1) + ': ' + Strings.encodeTags(dna.name);
					var selected = i < 24-numItems;
					var option = new Option(name, dna.id, selected, selected);
					option.dna = dna;
					sourceItemList.options[sourceItemList.length] = option;
				}
			}
			exreg.sourceItemsOnChange();
		}
		else
		{
			Wizard.setFatalError('No items available for processing.');
			return;
		}
*/

		Doc.show('step-1');
		Doc.show('gonext');
		Doc.hide('btnFakeNanoDropFile');
		Doc.hide('btnFakeDnaInput');
		Doc.hide('btnFakeQPcrFile');
		Doc.hide('extractSourceItems');
	}

/*
	exreg.sourceItemsOnChange = function()
	{
		sourceItemListIsValid = false;
		Wizard.setInputStatus('extractSourceItems');
		
		var frm = document.forms['meludi'];
		var sourceItemList = frm.extractSourceItems;
		
		var numSelected = 0;
		for (var i = 0; i < sourceItemList.length; i++)
		{
			if (sourceItemList[i].selected) numSelected++;
		}

		if (numSelected < 1 || numSelected > 24)
		{
			Wizard.setInputStatus('extractSourceItems', 'invalid', 'Must select 1-24 items.');
			return;
		}
		
		sourceItemListIsValid = true;
		Wizard.setInputStatus('extractSourceItems', 'valid');
	}
*/
	
	exreg.validateStep1 = function(event)
	{
/*
		if (!sourceItemListIsValid)
		{
			event.preventDefault();
		}
*/
		var frm = document.forms['meludi'];
		var startPlate = null;
		for (var i = 0; i < frm.startPlate.length; i++)
		{
			if (frm.startPlate[i].selected)
			{
				startPlate = frm.startPlate[i];
			}
		}
		if (startPlate == null)
		{
			//alert("exreg.validateStep1(): startPlate == null");
			event.preventDefault();
		}
		//alert("exreg.validateStep1(): startPlate.text = " + startPlate.text + " startPlate.value = " + startPlate.value);

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetSourceItemListFromStartPlate';
		url += '&startPlateId='+startPlate.value;
		url += '&includeSpecimenExtracts=true';
		Wizard.asyncJsonRequest(url, exreg.sourceItemListLoaded);
	}
	
	exreg.sourceItemListLoaded = function(response)
	{
		var frm = document.forms['meludi'];

		var startPlate = response.startplate;
		var specimens = response.specimens;
		var rnas = response.rnas;
		var dnas = response.dnas;
		var items = response.items;
		extraItemsList = response.extraitems;
		//alert("exreg.sourceItemListLoaded(): specimens.length = " + specimens.length + " rnas.length = " + rnas.length + " dnas.length = " + dnas.length);

		if (startPlate)
		{
			frm.startPlateName.value = startPlate.name;
			frm.startPlateId.value = startPlate.id;
			// Extraction parameters
			frm.isolationDate.value = startPlate.isolationDate;
			frm.extractionProtocolId.value = startPlate.extractionProtocolId;
			frm.extractionProtocolName.value = startPlate.extractionProtocolName;
/*
			frm.rnaDefaultVolume.value = startPlate.rnaDefaultVolume;
			frm.dnaDefaultVolume.value = startPlate.dnaDefaultVolume;
*/
			// QIAcube parameters
			frm.qiacubeDate.value = startPlate.qiacubeDate;
			frm.qiacubePrimaryPrefix.value = startPlate.qiacubePrimaryPrefix;
			frm.qiacubeSecondaryPrefix.value = startPlate.qiacubeSecondaryPrefix;
			frm.qiacubeRunNo.value = startPlate.qiacubeRunNo;
			frm.qiacubeOperator.value = startPlate.qiacubeOperator;
			frm.allPrepFfpeKit.value = startPlate.allPrepFfpeKit;
			frm.numExtraQiacubeItems.value = startPlate.numExtraQiacubeItems;

			frm.startPlateName.disabled = true;
			frm.isolationDate.disabled = true;
/*
			frm.rnaDefaultVolume.disabled = true;
			frm.dnaDefaultVolume.disabled = true;
*/
			// QIAcube parameters
			frm.qiacubeDate.disabled = true;
			frm.qiacubePrimaryPrefix.disabled = true;
			frm.qiacubeSecondaryPrefix.disabled = true;
			frm.qiacubeRunNo.disabled = true;
			frm.qiacubeOperator.disabled = true;
			frm.allPrepFfpeKit.disabled = true;
			frm.numExtraQiacubeItems.disabled = true;
/*
			alert("exreg.sourceItemListLoaded(): startPlate.isolationDate = " + startPlate.isolationDate);
			alert("exreg.sourceItemListLoaded(): startPlate.ExtractionProtocolName = " + startPlate.ExtractionProtocolName);
			//alert("exreg.sourceItemListLoaded(): startPlate.rnaDefaultVolume = " + startPlate.rnaDefaultVolume);
			///alert("exreg.sourceItemListLoaded(): startPlate.dnaDefaultVolume = " + startPlate.dnaDefaultVolume);

			alert("exreg.sourceItemListLoaded(): startPlate.qiacubeDate = " + startPlate.qiacubeDate);
			alert("exreg.sourceItemListLoaded(): startPlate.qiacubePrimaryPrefix = " + startPlate.qiacubePrimaryPrefix);
			alert("exreg.sourceItemListLoaded(): startPlate.qiacubeSecondaryPrefix = " + startPlate.qiacubeSecondaryPrefix);
			alert("exreg.sourceItemListLoaded(): startPlate.qiacubeRunNo = " + startPlate.qiacubeRunNo);
			alert("exreg.sourceItemListLoaded(): startPlate.qiacubeOperator = " + startPlate.qiacubeOperator);
			alert("exreg.sourceItemListLoaded(): startPlate.allPrepFfpeKit = " + startPlate.allPrepFfpeKit);
			alert("exreg.sourceItemListLoaded(): startPlate.numExtraQiacubeItems = " + startPlate.numExtraQiacubeItems);
*/
		}
		//alert("exreg.sourceItemListLoaded(): Startplate parameters loaded");

		var sourceItemList = frm.extractSourceItems;
		if (items.length)
		{
			var numItems = items.length;
			for (var i=0; i < items.length; i++)
			{
				var item = items[i];
				var name = (i+1) + ': ' + Strings.encodeTags(item.name);
				if (item.bioWell)
				{
					name += ' -- ' + Strings.encodeTags(item.bioWell.bioPlate.name + ' (' + item.bioWell.location + ')');
				}
				var selected = i < 24;
				// Identification of item via id works here as specimens and extracts
				// are stored in the same database table using discriminators, and are
				// therefore guaranteed to not have the same id value as another item in the table.
				var option = new Option(name, item.id, selected, selected);
				option.item = item;
				sourceItemList.options[sourceItemList.length] = option;
			}
		}
		else
		{
			Wizard.setFatalError('No items available for processing.');
			return;
		}
		//alert("exreg.sourceItemListLoaded(): End");
	}
	
	exreg.initializeStep2 = function()
	{
		var frm = document.forms['meludi'];
/*
		frm.extractSourceItems.disabled = true;
*/
		var startPlate = null;
		for (var i = 0; i < frm.startPlate.length; i++)
		{
			if (frm.startPlate[i].selected)
			{
				startPlate = frm.startPlate[i];
			}
		}
		if (startPlate == null)
		{
			//alert("exreg.validateStep1(): startPlate == null");
			event.preventDefault();
		}

		frm.startPlate.disabled = true;
		frm.rnaDefaultVolume.value = DEFAULT_RNA_VOLUME;
		frm.dnaDefaultVolume.value = DEFAULT_DNA_VOLUME;

		var clearButtons = Doc.element('commonInfoSection').getElementsByClassName('clear-reagents');
		if (clearButtons.length > 0)
		{
			Buttons.addClickHandler(clearButtons[0], exreg.clearReagents);
		}
		var copyButtons = Doc.element('commonInfoSection').getElementsByClassName('copy-reagents-from-latest-extract');
		if (copyButtons.length > 0)
		{
			Buttons.addClickHandler(copyButtons[0], exreg.copyReagentsFromLatestExtract);
		}

		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('gonext');
		Doc.hide('btnFakeNanoDropFile');
		Doc.hide('btnFakeDnaInput');
		Doc.hide('btnFakeQPcrFile');
		Doc.hide('startPlateId');
		Doc.hide('extractionProtocolId');

/*
		Meludi.loadProtocols('SAMPLE_HANDLING_PROTOCOL', 'extractionProtocol');
		Doc.addClass('extractionProtocol', 'list-loading');
		frm.extractionProtocol[0] = new Option('loading...', '');
		var url = '../Protocol.servlet?ID='+App.getSessionId();
		url += '&cmd=GetProtocols&subtype=EXTRACTION_PROTOCOL';
		Wizard.asyncJsonRequest(url, exreg.protocolsLoaded);
*/

		frm.isolationDate.focus();
		Events.sendChangeEvent('isolationDate');
		Events.sendChangeEvent('qiacubeDate');
	}

/*	
	exreg.protocolsLoaded = function(response)
	{
		var frm = document.forms['meludi'];
		var protocols = response.protocols;

		Doc.removeClass('extractionProtocol', 'list-loading');
		frm.extractionProtocol.length = 0;
		for (var i = 0; i < protocols.length; i++)
		{
			var name = protocols[i].name;
			var id = protocols[i].id;
			protocolMap[id] = name;
			frm.extractionProtocol[frm.extractionProtocol.length] = new Option(name, id, false, true);
		}
		frm.extractionProtocol[frm.extractionProtocol.length] = new Option('- none -', '');

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLastProcessedDna';
		Wizard.asyncJsonRequest(url, exreg.lastProcessedDnaLoaded);
	}
*/

	exreg.clearReagents = function()
	{
		var frm = document.forms['meludi'];

		// Clear reagent values

		// Specimen reagents
		frm.xyleneSpecimen.value = null;
		frm.etoh995Specimen.value = null;
		frm.bufferPkdSpecimen.value = null;
		frm.bufferAtlSpecimen.value = null;
		frm.proteinaseKSpecimen.value = null;
		frm.rnaseASpecimen.value = null;

		// QIAcube reagents
		frm.etoh995.value = null;
		frm.bufferRlt.value = null;
		frm.bufferFrn.value = null;
		frm.bufferRpe.value = null;
		frm.bufferAl.value = null;
		frm.bufferAw1.value = null;
		frm.bufferAw2.value = null;
		frm.bufferAte.value = null;
		frm.bufferRdd.value = null;
		frm.rnaseFreeWater.value = null;
		frm.dnaseMix.value = null;
		frm.rneasyMinEluteSpinColumn.value = null;
		frm.qiaampMinEluteSpinColumn.value = null;
	}

	exreg.copyReagentsFromLatestExtract = function()
	{
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLastProcessedDna';
		Wizard.asyncJsonRequest(url, exreg.lastProcessedDnaLoaded);
	}

	exreg.lastProcessedDnaLoaded = function(response)
	{
		var frm = document.forms['meludi'];
		var dnas = response.dnas;

		if (dnas.length > 0)
		{
			var dna = dnas[0];

			// Reset the existing values

			// AllPrep FFPE Kit			
			var allPrepFfpeKit = '';

			// Specimen reagents
			var specXylene = '';
			var specEtoh995 = '';
			var specBufferPkd = '';
			var specBufferAtl = '';
			var specProteinaseK = '';
			var specRNAseA = '';

			// QIAcube reagents
			var bufferRlt = '';
			var etoh995 = '';
			var bufferFrn = '';
			var bufferRpe = '';
			var rnaseFreeWater = '';
			var dnaseMix = '';
			var bufferRdd = '';
			var rneasyMinEluteSpinColumn = '';
			var bufferAl = '';
			var bufferAW1 = '';
			var bufferAW2 = '';
			var bufferAte = '';
			var qiaampMinEluteSpinColumn = '';

			if (dna) 
			{
				// AllPrep FFPE Kit			
				allPrepFfpeKit = dna.allPrepFfpeKit;

				// Specimen reagents
				specXylene = dna.xyleneSpecimen;
				specEtoh995 = dna.etoh995Specimen;
				specBufferPkd = dna.bufferPkdSpecimen;
				specBufferAtl = dna.bufferAtlSpecimen;
				specProteinaseK = dna.proteinaseKSpecimen;
				specRNAseA = dna.rnaseASpecimen;

				// QIAcube reagents
				bufferRlt = dna.bufferRlt;
				etoh995 = dna.etoh995;
				bufferFrn = dna.bufferFrn;
				bufferRpe = dna.bufferRpe;
				rnaseFreeWater = dna.rnaseFreeWater;
				dnaseMix = dna.dnaseMix;
				bufferRdd = dna.bufferRdd;
				rneasyMinEluteSpinColumn = dna.rneasyMinEluteSpinColumn;
				bufferAl = dna.bufferAl;
				bufferAW1 = dna.bufferAw1;
				bufferAW2 = dna.bufferAw2;
				bufferAte = dna.bufferAte;
				qiaampMinEluteSpinColumn = dna.qiaampMinEluteSpinColum;
			}

			// Update reagent values

			// AllPrep FFPE Kit
			if (frm.allPrepFfpeKit.value == null || frm.allPrepFfpeKit.value == '')
			{
				frm.allPrepFfpeKit.value = allPrepFfpeKit;
			}

			// Specimen reagents
			frm.xyleneSpecimen.value = specXylene;
			frm.etoh995Specimen.value = specEtoh995;
			frm.bufferPkdSpecimen.value = specBufferPkd;
			frm.bufferAtlSpecimen.value = specBufferAtl;
			frm.proteinaseKSpecimen.value = specProteinaseK;
			frm.rnaseASpecimen.value = specRNAseA;

			// QIAcube reagents
			frm.etoh995.value = etoh995;
			frm.bufferRlt.value = bufferRlt;
			frm.bufferFrn.value = bufferFrn;
			frm.bufferRpe.value = bufferRpe;
			frm.bufferAl.value = bufferAl;
			frm.bufferAw1.value = bufferAW1;
			frm.bufferAw2.value = bufferAW2;
			frm.bufferAte.value = bufferAte;
			frm.bufferRdd.value = bufferRdd;
			frm.rnaseFreeWater.value = rnaseFreeWater;
			frm.dnaseMix.value = dnaseMix;
			frm.rneasyMinEluteSpinColumn.value = rneasyMinEluteSpinColumn;
			frm.qiaampMinEluteSpinColumn.value = qiaampMinEluteSpinColumn;
		}
	}

	exreg.qiacubeDateOnChange = function()
	{
		var frm = document.forms['meludi'];
		
		var isolationDateIsValid = Wizard.isValid('isolationDate');
		var qiacubeDateIsValid = Wizard.isValid('qiacubeDate');
		
		if (!isolationDateIsValid || !qiacubeDateIsValid) return;

		var isolationDate = Dates.parseString(frm.isolationDate.value, 'yyyyMMdd');
		var qiacubeDate = Dates.parseString(frm.qiacubeDate.value, 'yyyyMMdd');
		if (isolationDate > qiacubeDate)
		{
			Wizard.setInputStatus('qiacubeDate', 'invalid', 'QIAcube date is before isolation date');
			qiacubeDateIsAfterIsolationDate = false;
			return;
		}

		qiacubeDateIsAfterIsolationDate = true;
		
		Wizard.setInputStatus('qiacubeDate', 'valid');
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextQiacubeRunNo&qiacubeDate='+encodeURIComponent(frm.qiacubeDate.value);
		Wizard.showLoadingAnimation('Loading Qiacube run number...');
		Wizard.asyncJsonRequest(url, exreg.gotNextRunNumber);
	}

	exreg.gotNextRunNumber = function(response)
	{
		var frm = document.forms['meludi'];
		frm.qiacubeRunNo.value = response.nextQiacubeRunNo;
		Events.sendChangeEvent('qiacubeRunNo');
	}

	exreg.qiacubeRunNoOnChange = function()
	{
		var frm = document.forms['meludi'];
		var runNumber = frm.qiacubeRunNo.value;
		qiacubeRunNoIsValid = false;

		if (runNumber == '')
		{
			Wizard.setInputStatus('qiacubeRunNo', 'invalid', 'Missing');
			return;
		}
		if (!(parseInt(runNumber, 10) >= 1))
		{
			Wizard.setInputStatus('qiacubeRunNo', 'invalid', 'Must be at least 1');
			return;		
		}
		
		Wizard.setInputStatus('qiacubeRunNo', 'valid');
		qiacubeRunNoIsValid = true;
	}

	
	exreg.reagentFileOnChange = function()
	{
		var frm = document.forms['meludi'];
		var file = frm.reagentFile.files[0];
		
		var reader = new FileReader();
		reader.onload = exreg.reagentFileLoaded;
		reader.readAsText(file);
	}

	exreg.reagentFileLoaded = function(event)
	{
		exreg.parseReagentFile(event.target.result);
	}

	exreg.parseReagentFile = function(data)
	{
		var frm = document.forms['meludi'];
		reagentFileIsValid = false;

		var numReagentItems = 0;
		// Reset the existing values

		// AllPrep FFPE Kit			
		var allPrepFfpeKit = '';

		// Specimen reagents
		var specXylene = '';
		var specEtoh995 = '';
		var specBufferPkd = '';
		var specBufferAtl = '';
		var specProteinaseK = '';
		var specRNAseA = '';

		// QIAcube reagents
		var bufferRlt = '';
		var etoh995 = '';
		var bufferFrn = '';
		var bufferRpe = '';
		var rnaseFreeWater = '';
		var dnaseMix = '';
		var bufferRdd = '';
		var rneasyMinEluteSpinColumn = '';
		var bufferAl = '';
		var bufferAW1 = '';
		var bufferAW2 = '';
		var bufferAte = '';
		var qiaampMinEluteSpinColumn = '';

		// Find column separator as first character after first occurrence of search string
		var separator = exreg.findSeparator(data, "Specimen Reagents");
		if (separator == null)
		{
			// Comma is default separator
			separator = ',';
		}
		var columnSplitRegexp = new RegExp(" *" + separator + " *",'');
		var lines = data.split(/[\n\r]+/);
		var columns = lines[0].split(columnSplitRegexp);

		var numImported = 0;
		var inQiaCubeSection = false;
		for (var i = 1; i < lines.length; i++)
		{
			if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
			var row = lines[i].split(columnSplitRegexp);
			if (row[0] == 'Qiacube Reagents')
			{
				inQiaCubeSection = true;
			}
			if (row.length > 1) 
			{
				var key = row[0];
				if (!key || key == '') continue; // with the next line

				// AllPrep FFPE Kit			
				if (key == 'Specimen Reagents')
				{
					allPrepFfpeKit = row[1];
				}

				// Specimen reagents
				if (key == 'Xylen')
				{
					specXylene = row[1];
				}
				if (key == 'ETOH 99,5%')
				{
					if (!inQiaCubeSection)
					{
						specEtoh995 = row[1];
					}
				}
				if (key == 'Buffer PKD')
				{
					specBufferPkd = row[1];
				}
				if (key == 'Buffer ATL')
				{
					specBufferAtl = row[1];
				}
				if (key == 'Proteinase K')
				{
					specProteinaseK = row[1];
				}
				if (key == 'Rnase A')
				{
					specRNAseA = row[1];
				}

				// QIAcube reagents
				if (key == 'Buffer RLT')
				{
					bufferRlt = row[1];
				}
				if (key == 'ETOH 99,5%')
				{
					if (inQiaCubeSection)
					{
						etoh995 = row[1];
					}
				}
				if (key == 'Buffer FRN')
				{
					bufferFrn = row[1];
				}
				if (key == 'Buffer RPE')
				{
					bufferRpe = row[1];
				}
				if (key == 'Rnase-free Water')
				{
					rnaseFreeWater = row[1];
				}
				if (key == 'DNAse mix')
				{
					dnaseMix = row[1];
				}
				if (key == 'RDD')
				{
					bufferRdd = row[1];
				}
				if (key == 'Rneasy MinElute Spin Column')
				{
					rneasyMinEluteSpinColumn = row[1];
				}
				if (key == 'Buffer AL')
				{
					bufferAl = row[1];
				}
				if (key == 'Buffer AW1')
				{
					bufferAW1 = row[1];
				}
				if (key == 'Buffer AW2')
				{
					bufferAW2 = row[1];
				}
				if (key == 'Buffer ATE')
				{
					bufferAte = row[1];
				}
				if (key == 'QIAamp MinElute Spin Column')
				{
					qiaampMinEluteSpinColumn = row[1];
				}
			}
		}

		// Update reagent values

		// Specimen reagents
		frm.xyleneSpecimen.value = specXylene;
		frm.etoh995Specimen.value = specEtoh995;
		frm.bufferPkdSpecimen.value = specBufferPkd;
		frm.bufferAtlSpecimen.value = specBufferAtl;
		frm.proteinaseKSpecimen.value = specProteinaseK;
		frm.rnaseASpecimen.value = specRNAseA;

		// QIAcube reagents
		if (frm.allPrepFfpeKit.value == null || frm.allPrepFfpeKit.value == '')
		{
			frm.allPrepFfpeKit.value = allPrepFfpeKit;
		}
		frm.etoh995.value = etoh995;
		frm.bufferRlt.value = bufferRlt;
		frm.bufferFrn.value = bufferFrn;
		frm.bufferRpe.value = bufferRpe;
		frm.bufferAl.value = bufferAl;
		frm.bufferAw1.value = bufferAW1;
		frm.bufferAw2.value = bufferAW2;
		frm.bufferAte.value = bufferAte;
		frm.bufferRdd.value = bufferRdd;
		frm.rnaseFreeWater.value = rnaseFreeWater;
		frm.dnaseMix.value = dnaseMix;
		frm.rneasyMinEluteSpinColumn.value = rneasyMinEluteSpinColumn;
		frm.qiaampMinEluteSpinColumn.value = qiaampMinEluteSpinColumn;

		if (numImported != numReagentItems)
		{
			Wizard.setInputStatus('reagentFile', 'invalid', 'Could not find values for all ' + numReagentItems + ' samples: ' + numImported);
			return;
		}

		Wizard.setInputStatus('reagentFile', 'valid');
		reagentFileIsValid = true;
	}

	exreg.findReagentColumn = function(columns, col)
	{
		var colIndex = columns.indexOf(col);
		if (colIndex == -1)
		{
			Wizard.setInputStatus('reagentFile', 'invalid', 'On line 1; Can\'t find "' + col + '" column');
		}
		return colIndex;
	}

	exreg.validateStep2 = function(event)
	{
		var valid = true;
		valid &= Wizard.isValid('isolationDate');
		valid &= Wizard.isValid('qiacubeDate');
		valid &= qiacubeRunNoIsValid;
		
		if (!valid) event.preventDefault();
	}

	
	exreg.initializeStep3 = function()
	{
		var frm = document.forms['meludi'];
		var startPlateName = frm.startPlateName.value;
		Doc.element('detailsStartPlateName').innerHTML = '<b>Start plate: ' + startPlateName + '</b>';
		Doc.element('qubitFileDilutionFactor').value = QUBIT_DILUTION_FACTOR_DEFAULT;
		var rnaDefaultVolume = exreg.getNumber(frm.rnaDefaultVolume.value);
		var dnaDefaultVolume = exreg.getNumber(frm.dnaDefaultVolume.value);
		qiacubeSortedPrefixes = exreg.getQiacubeSortedPrefixes();
		
		var html = '<table id="detailsTable" class="step-form">';
		// First header row
		html += '<thead>';
		html += '<tr>';
		html += '<th>Extract</th>';
		html += '<th class="dottedleft">QIAcube</th>';
		html += '<th colspan="1" class="dottedleft"></th>';
/*
		html += '<th colspan="1" class="dottedleft"></th>';
*/
/*
		html += '<th colspan="5" class="dottedleft">RNA</th>';
*/
/*
		html += '<th colspan="3" class="dottedleft">RNA</th>';
		html += '<th colspan="4" class="dottedleft">DNA</th>';
*/
		html += '<th colspan="5" class="dottedleft">RNA</th>';
		html += '<th colspan="5" class="dottedleft">DNA</th>';
		html += '<th></th>';
		html += '</tr>';
		// Second header row
		html += '<tr>';
		html += '<th>source item</th>';
		html += '<th class="dottedleft">position</th>';
		html += '<td class="dottedleft">Storage</td>';
/*
		html += '<td class="dottedleft">Pos</td><td>Vol.</td><td>Conc.</td><td>Quantity</td><td><span id="toggle-normalize-1" class="link" title="Toggle all">Pre-</span></td>';
*/
		html += '<td class="dottedleft">NanoDrop</td><td>RNA</td><td>Vol.</td><td>Conc.</td><td>Quantity</td>';
		html += '<td class="dottedleft">DNA</td><td>Vol.</td><td>Conc.</td><td>Quantity</td><td>ΔCt</td>';
		html += '<td class="dottedleft comment"></td>';
		html += '<td></td>';
		html += '</tr>';
		// Third header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<td class="dottedleft">box</td>';
/*
		html += '<td class="dottedleft"></td><td>(µl)</td><td>(ng/µl)</td><td>(µg)</td><td><span id="toggle-normalize-2" class="link" title="Toggle all">normalize</span></td>';
*/
		html += '<td class="dottedleft">Pos</td><td>Plate Pos.</td><td>(µl)</td><td>(ng/µl)</td><td>(µg)</td>';
		html += '<td class="dottedleft">Plate Pos.</td><td>(µl)</td><td>(ng/µl)</td><td>(µg)</td><td></td>';
		html += '<td class="dottedleft comment"></td>';
		html += '<td></td>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';

		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numSpecimenItems = exreg.getNumberOfSpecimenItems();
		var numRnaItems = exreg.getNumberOfRnaItems();
		var numDnaItems = exreg.getNumberOfDnaItems();
		var numNanoDropItems = numSpecimenItems + numRnaItems;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		var numQiacubeItems = numSpecimenItems + numExtraQiacubeItems;
		var totNumItems = numItems + numExtraQiacubeItems;
		var itemNo = 0;
		var qiacubeItemNo = 0;
		// Set default values for DNA/RNA extraction
		for (var i = 0; i < totNumItems; i++)
		{
			itemNo++;
			var item = null;
			var storageBox = '-';
			var rnaStorageBox = '-';
			var dnaStorageBox = '-';
			if (i < numItems)
			{
				item = selItemsList[i];
				var qiacubePos = null;
				if (exreg.isSpecimen(item))
				{
					qiacubeItemNo++;
					qiacubePos = exreg.getDetailedQiaCubePosition(numQiacubeItems, qiacubeItemNo);
				}
/*
				item.originalQuantity = DEFAULT_SPECIMEN_VOLUME;
				item.processedQuantity = DEFAULT_SPECIMEN_USED;
*/
				item.qiacubePosition = qiacubePos;
				item.originalQiacubePosition = qiacubePos;
				
				var rna = {};
				var dna = {};
				// RNA
				rna.name = item.name + '.r';
				rna.volume = rnaDefaultVolume;
				// DNA
				dna.name = item.name + '.d';
				dna.volume = dnaDefaultVolume;
/*				
				item.rna = rna;
				item.dna = dna;
*/
				if (!item.rna)
				{
					item.rna = rna;
				}
				else if (!item.rna.volume)
				{
					item.rna.volume = rnaDefaultVolume;
				}
				if (!item.dna)
				{
					item.dna = dna;
				}
				else if (!item.dna.volume)
				{
					item.dna.volume = dnaDefaultVolume;
				}
/*
				var storageBox = '-';
*/
				// Input extract source item specimen/RNA/DNA
				if (item.bioWell != null)
				{
					storageBox = Strings.encodeTags(item.bioWell.bioPlate.name + '[' + item.bioWell.location + ']');
				}
				// RNA extract from specimen
				if (item.rna && item.rna.bioWell)
				{
					rnaStorageBox = Strings.encodeTags(item.rna.bioWell.bioPlate.name + '[' + item.rna.bioWell.location + ']');
				}
				// DNA extract from specimen
				if (item.dna && item.dna.bioWell)
				{
					dnaStorageBox = Strings.encodeTags(item.dna.bioWell.bioPlate.name + '[' + item.dna.bioWell.location + ']');
				}
				// Input RNA item storage location should be displayed in the RNA extract storage location column
				if (exreg.isRna(item) && item.bioWell != null)
				{
					rnaStorageBox = Strings.encodeTags(item.bioWell.bioPlate.name + '[' + item.bioWell.location + ']');
				}
				// Input DNA item storage location should be displayed in the DNA extract storage location column
				if (exreg.isDna(item) && item.bioWell != null)
				{
					dnaStorageBox = Strings.encodeTags(item.bioWell.bioPlate.name + '[' + item.bioWell.location + ']');
				}
			}
			else
			{
				// Extra QIAcube item
				item = extraItemsList[i - numItems];
				qiacubeItemNo++;
				var qiacubePos = exreg.getDetailedQiaCubePosition(numQiacubeItems, qiacubeItemNo);
				item.qiacubePosition = qiacubePos;
				item.originalQiacubePosition = qiacubePos;
			}
			html += '<tr class="highlight">';
			html += '<th>' + Strings.encodeTags(item.name) + '</th>';
			html += '<td id="qiacubePosition'+i+'" class="dottedleft"></td>';
			html += '<td class="dottedleft">'+storageBox+'</td>';
			html += '<td id="rnaPos'+i+'" class="dottedleft"></td>';
			html += '<td id="rnaPlatePos'+i+'" class="dottedleft">'+rnaStorageBox+'</td>';
			html += '<td id="rnaVol'+i+'"></td>';
			if (exreg.isSpecimen(item) || exreg.isRna(item))
			{
				html += '<td class="bg-filled-50 italic"><input type="text" id="rnaConc'+i+'" style="width: 5em;"></intput></td>';
			}
			else
			{
				html += '<td id="rnaConc'+i+'" class="bg-filled-50 italic"></td>';
			}
			//html += '<td id="rnaConc'+i+'" class="bg-filled-50 italic"></td>';
			html += '<td id="rnaQuantity'+i+'" class="bg-filled-50 italic"></td>';
/*
			if (exreg.isSpecimen(item) || exreg.isRna(item))
			{
				html += '<td><input type="checkbox" name="rnaPreNormalize'+i+'" id="rnaPreNormalize'+i+'" disabled>';
			}
			else
			{
				html += '<td id="rnaPreNormalize'+i+'">-</td>';
			}
*/
/*
			html += '<span id="rnaPreNormalize'+i+'.status" class="status">&nbsp;</span></td>';
*/
			html += '<td id="dnaPlatePos'+i+'" class="dottedleft">'+dnaStorageBox+'</td>';
			html += '<td id="dnaVol'+i+'"></td>';
			if (exreg.isSpecimen(item) || exreg.isDna(item))
			{
				html += '<td class="bg-filled-50 italic"><input type="text" id="dnaConc'+i+'" style="width: 5em;"></intput></td>';
			}
			else
			{
				html += '<td id="dnaConc'+i+'"  class="bg-filled-50 italic">-</td>';
			}
			html += '<td id="dnaQuantity'+i+'" class="bg-filled-50 italic"></td>';
			if (exreg.isSpecimen(item) || exreg.isDna(item))
			{
				html += '<td class="bg-filled-50 italic"><input type="text" id="deltaCt'+i+'" style="width: 5em;"></intput></td>';
			}
			else
			{
				html += '<td id="deltaCt'+i+'" class="bg-filled-50 italic"></td>';
			}
			//html += '<td id="deltaCt'+i+'" class="bg-filled-50 italic"></td>';
			html += '<td id="comments'+i+'" class="dottedleft comment"></td>';
			html += '<td><div class="button basicbutton interactable edit-details" style="width: 5em;" data-index="'+i+'">Edit&hellip;</div></td>';
			html += '</tr>';
		}
/*
		if (numSpecimenItems == 1 || numSpecimenItems == 11 || numSpecimenItems == 23)
*/
		if (numQiacubeItems == 1 || numQiacubeItems == 11 || numQiacubeItems == 23)
		{
			// Add extra row for counterbalance in QIAcube
			counterBalanceInfoRow = true;
			var i = numItems;
/*
			var qiacubePos = exreg.getDetailedQiaCubePosition(numSpecimenItems + 1, numSpecimenItems + 1);
*/
			var qiacubePos = exreg.getDetailedQiaCubePosition(numQiacubeItems + 1, numQiacubeItems + 1);
				
			html += '<tr class="highlight">';
			html += '<th>' + Strings.encodeTags('COUNTER-BALANCE') + '</th>';
			html += '<td id="qiacubePosition'+i+'" class="dottedleft">'+qiacubePos+'</td>';
			html += '<td class="dottedleft">-</td>';
			html += '<td id="rnaPos'+i+'" class="dottedleft">-</td>';
			html += '<td id="rnaPlatePos'+i+'" class="dottedleft">-</td>';
			html += '<td id="rnaVol'+i+'">-</td>';
			html += '<td id="rnaConc'+i+'" class="bg-filled-50 italic">-</td>';
			html += '<td id="rnaQuantity'+i+'" class="bg-filled-50 italic">-</td>';
/*
			html += '<td>-</td>';
*/
			html += '<td id="dnaPlatePos'+i+'" class="dottedleft">-</td>';
			html += '<td id="dnaVol'+i+'">-</td>';
			html += '<td id="dnaConc'+i+'" class="bg-filled-50 italic">-</td>';
			html += '<td id="dnaQuantity'+i+'" class="bg-filled-50 italic">-</td>';
			html += '<td id="deltaCt'+i+'" class="bg-filled-50 italic">-</td>';
			html += '<td id="comments'+i+'" class="dottedleft comment">Reminder to use counterbalance</td>';
			html += '<td>-</td>';
			html += '</tr>';				
		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('detailsSection').innerHTML = html;
		for (var i = 0; i < numItems; i++)
		{
			Events.addEventHandler('rnaConc'+i, 'change', exreg.rnaConcOnChange);
			Events.addEventHandler('dnaConc'+i, 'change', exreg.dnaConcOnChange);
			Events.addEventHandler('deltaCt'+i, 'change', exreg.deltaCtOnChange);
		}
		var buttons = Doc.element('detailsSection').getElementsByClassName('edit-details');
		for (var i = 0; i < buttons.length; i++)
		{
			Buttons.addClickHandler(buttons[i], exreg.editDnaRnaDetails);
		}

/*		
		Events.addEventHandler('toggle-normalize-1', 'click', exreg.toggleNormalize);
		Events.addEventHandler('toggle-normalize-2', 'click', exreg.toggleNormalize);
*/
		
		exreg.updateDetailsTable();
		
		Wizard.setCurrentStep(3);
		Doc.show('gocancel');
		Doc.show('gocreate');
		Doc.show('goregister');
		Doc.show('btnFakeNanoDropFile');
		Doc.show('btnFakeDnaInput');
		Doc.show('btnFakeQPcrFile');
		
		//Wizard.setInputStatus('nanoDropFile', 'invalid', 'No file selected');
		Wizard.keepSessionAlive();
		
		if (debug && location.protocol != 'https:') 
		{
			Doc.show('debug-tools');
		}
	}

	exreg.getQiacubeSortedPrefixes = function()
	{
		var frm = document.forms['meludi'];
		var primaryQiacubePrefix = frm.qiacubePrimaryPrefix.value;
		var secondaryQiacubePrefix = frm.qiacubeSecondaryPrefix.value;

		// Get string of current QIAcube prefixes in order
		var fullSortedPrefixSet = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
		var sortedPrefixes = '' + primaryQiacubePrefix + secondaryQiacubePrefix;
		for (var i = 0; i < fullSortedPrefixSet.length; i++)
		{
			// Get next prefix
			var prefix = fullSortedPrefixSet.substring(i, i+1);
			// Add prefix to string of sorted prefixes, if not already present
			if (prefix != primaryQiacubePrefix && prefix != secondaryQiacubePrefix)
			{
				sortedPrefixes += prefix;
			}
		}
		return sortedPrefixes;
	}

	exreg.getDetailedQiaCubePosition = function(nitems, i)
	{
		var dpos = null;
		var qiacubePos = null;
		// Two QIAcube instruments are available, denoted 'C' (default) and 'A'
		var qiacubeId = qiacubeSortedPrefixes.substring(0, 1);
		if (nitems == null || nitems < 1 || nitems > 24)
		{
			return null;
		}
		if (i == null || i < 1 || i > nitems)
		{
			return null;
		}
		if (nitems <= 12)
		{
			qiacubePos = exreg.getQiaCubePosition(nitems, i);
		}
		else
		{
			var numItemsQiaCube1 = Math.floor((nitems + 1)/2);
			if (numItemsQiaCube1 == 11)
			{
				numItemsQiaCube1 = 12;
			}
			if (i <= numItemsQiaCube1)
			{
				// Place sample in first QIAcube
				qiacubePos = exreg.getQiaCubePosition(numItemsQiaCube1, i);
			}
			else
			{
				// Place sample in second QIAcube
				var numItemsQiaCube2 = nitems - numItemsQiaCube1;
				qiacubePos = exreg.getQiaCubePosition(numItemsQiaCube2, i - numItemsQiaCube1);
				qiacubeId = qiacubeSortedPrefixes.substring(1, 2);
			}
		}
		if (qiacubeId != null && qiacubePos != null)
		{
			dpos = qiacubeId + qiacubePos;
		}
		return dpos;
	}

	exreg.getQiaCubePosition = function(nitems, i)
	{
		var pos = null;
		if (nitems == null || nitems < 1 || nitems > 12)
		{
			return null;
		}
		if (i == null || i < 1 || i > nitems)
		{
			return null;
		}
		var posArr = ['1','2','3','4','5','6','7','8','9','10','11','12'];
		if (nitems == 2)
		{
			posArr = ['1','7','','','','','','','','','',''];
		}
		else if (nitems == 3)
		{
			posArr = ['1','5','9','','','','','','','','',''];
		}
		else if (nitems == 4)
		{
			posArr = ['1','2','7','8','','','','','','','',''];
		}
		else if (nitems == 5)
		{
			posArr = ['1','2','5','8','9','','','','','','',''];
		}
		else if (nitems == 6)
		{
			posArr = ['1','2','3','7','8','9','','','','','',''];
		}
		else if (nitems == 7)
		{
			posArr = ['1','2','3','6','7','9','10','','','','',''];
		}
		else if (nitems == 8)
		{
			posArr = ['1','2','3','4','7','8','9','10','','','',''];
		}
		else if (nitems == 9)
		{
			posArr = ['1','2','3','5','6','7','9','10','11','','',''];
		}
		else if (nitems == 10)
		{
			posArr = ['1','2','3','4','5','7','8','9','10','11','',''];
		}
		else if (nitems == 11)
		{
			// Counterbalance blank vial filled with water needed i position 12.
			posArr = ['1','2','3','4','5','6','7','8','9','10','11',''];
		}
		pos = posArr[i-1];
		return pos;
	}

	exreg.toggleNormalize = function()
	{
		var frm = document.forms['meludi'];
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Toggle normalize check box status
		var check = null;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exreg.isSpecimen(item) || exreg.isRna(item))
			{
				if (!frm['rnaPreNormalize'+i].disabled)
				{
					if (check == null) check = !frm['rnaPreNormalize'+i].checked;
					frm['rnaPreNormalize'+i].checked = check;
				}
			}
		}
	}
	
	exreg.getSelectedItemsList = function()
	{
		var frm = document.forms['meludi'];

		// Get number of selected extract source items
		var numItems = 0;
		var selItemsList = [];
		for (var i = 0; i < frm.extractSourceItems.length; i++)
		{
			if (frm.extractSourceItems[i].selected)
			{
				if (frm.extractSourceItems[i].item != null)
				{
					selItemsList[numItems] = frm.extractSourceItems[i].item;
					numItems++;
				}
			}
		}
		return selItemsList;
	}

	exreg.getExtraItemsList = function(numExtraItems)
	{
		// Create extra items
		for (var i = 0; i < numExtraItems; i++)
		{
			var no = i + 1;
			var numStr = '' + no;
			if (no < 10)
			{
				numStr = '0' + numStr;
			}
			var jsonString = '{"name":"' + EXTRA_QIACUBE_SAMPLE_NAME_START + numStr + '","qiacubePosition":"","originalQiacubePosition":""}';
			var extraItem = JSON.parse(jsonString);
			extraItemsList[i] = extraItem;
		}
		return extraItemsList;
	}

	exreg.isSpecimen = function(item)
	{
		var isSpecimen = false;
		if (item != null)
		{
			if (!exreg.isRna(item) && !exreg.isDna(item))
			{
				var name = item.name;
				if (name != null)
				{
					// Check if item name does not start with string EXTRA_QIACUBE_SAMPLE_NAME_START
					if (name.indexOf(EXTRA_QIACUBE_SAMPLE_NAME_START) != 0)
					{
						isSpecimen = true;
					}
				}
			}
		}
		return isSpecimen;
	}

	exreg.isRna = function(item)
	{
		var isRna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".r"
				if (name.indexOf('.r') == (name.length - 2))
				{
					isRna = true;
				}
			}
		}
		return isRna;
	}

	exreg.isDna = function(item)
	{
		var isDna = false;
		if (item != null)
		{
			var name = item.name;
			if (name != null)
			{
				// Check if item name ends with ".d"
				if (name.indexOf('.d') == (name.length - 2))
				{
					isDna = true;
				}
			}
		}
		return isDna;
	}

	exreg.getNumberOfSpecimenItems = function()
	{
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numSpecimen = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exreg.isSpecimen(item))
			{
				numSpecimen++;
			}
		}
		return numSpecimen;
	}
	
	exreg.getNumberOfRnaItems = function()
	{
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numRna = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exreg.isRna(item))
			{
				numRna++;
			}
		}
		return numRna;
	}
	
	exreg.getNumberOfDnaItems = function()
	{
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		var numDna = 0;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exreg.isDna(item))
			{
				numDna++;
			}
		}
		return numDna;
	}
	
	exreg.rnaConcOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Update RNA concentration values
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			var rnaConc = Doc.element('rnaConc'+i);
			if (rnaConc != null)
			{
				if (rnaConc.value != null && rnaConc.value != '' && rnaConc.value != '-')
				{
					item.rna.ndConc = exreg.getNumber(rnaConc.value);
				}
				else
				{
					item.rna.ndConc = null;
				}
			}
		}
		exreg.updateDetailsTable();
	}

	exreg.dnaConcOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Update DNA concentration values
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			var dnaConc = Doc.element('dnaConc'+i);
			if (dnaConc != null)
			{
				if (dnaConc.value != null && dnaConc.value != '' && dnaConc.value != '-')
				{
					item.dna.qubitConc = exreg.getNumber(dnaConc.value);
				}
				else
				{
					item.dna.qubitConc = null;
				}
			}
		}
		exreg.updateDetailsTable();
	}

	exreg.deltaCtOnChange = function(event)
	{
		var frm = document.forms['meludi'];
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Update DNA deltaCt values
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			var deltaCt = Doc.element('deltaCt'+i);
			if (deltaCt != null)
			{
				if (deltaCt.value != null && deltaCt.value != '' && deltaCt.value != '-')
				{
					item.dna.deltaCt = exreg.getNumber(deltaCt.value);
				}
				else
				{
					item.dna.deltaCt = null;
				}
			}
		}
		exreg.updateDetailsTable();
	}

	exreg.updateDetailsTable = function()
	{
		var frm = document.forms['meludi'];
		var rnaDefaultVolume = exreg.getNumber(frm.rnaDefaultVolume.value);
		var dnaDefaultVolume = exreg.getNumber(frm.dnaDefaultVolume.value);
		var hasQiacubePosition = [];
		var duplicatePosition = 0;

		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		var totNumItems = numItems + numExtraQiacubeItems;
		//var extraItemsList = exreg.getExtraItemsList(numExtraQiacubeItems);
		// Get sorted list of used integer QIAcube positions for specimens, generalized for more than one QIAcube
		var specSortedIntQiacubePosition = [];
		var qiacubeItemNo = 0;
		for (var i = 0; i < totNumItems; i++)
		{
			if (i < numItems)
			{

				var item = selItemsList[i];
				if (exreg.isSpecimen(item))
				{
					var intQiacubePosition = exreg.getIntQiacubePosition(item.qiacubePosition);
					specSortedIntQiacubePosition[qiacubeItemNo] = intQiacubePosition;
					qiacubeItemNo++;
				}
			}
			else
			{
				// Extra QIAcube item
				var item = extraItemsList[i - numItems];
				var intQiacubePosition = exreg.getIntQiacubePosition(item.qiacubePosition);
				specSortedIntQiacubePosition[qiacubeItemNo] = intQiacubePosition;
				qiacubeItemNo++;
			}
		}
		// Sort for ascending integer values
		specSortedIntQiacubePosition.sort(function(a,b){return a-b});
		var numSpecimenItems = exreg.getNumberOfSpecimenItems();
		var numRnaItems = exreg.getNumberOfRnaItems();
		var numDnaItems = exreg.getNumberOfDnaItems();
		var numNanoDropItems = numSpecimenItems + numRnaItems;
		var rnaItemNo = 0;
		// Set default values for DNA/RNA extraction
		for (var i = 0; i < totNumItems; i++)
		{
			var item = null;
			if (i < numItems)
			{
				item = selItemsList[i];
			}
			else
			{
				// Extra QIAcube item
				item = extraItemsList[i - numItems];
			}
			Doc.element('qiacubePosition'+i).innerHTML = item.qiacubePosition;
			Doc.addOrRemoveClass('qiacubePosition'+i, 'nondefault', item.qiacubePosition != item.originalQiacubePosition);

			var intQiacubePosition = exreg.getIntQiacubePosition(item.qiacubePosition);			
			if (intQiacubePosition != null)
			{
				if (hasQiacubePosition[intQiacubePosition])
				{
					duplicatePosition = intQiacubePosition;
				}
				else
				{
					hasQiacubePosition[intQiacubePosition] = true;
				}
			}
			var intNanoDropPlatePosition = intQiacubePosition;
			// Get integer NanoDrop position for specimens from index in sorted list of used integer QIAcube positions
			if (intQiacubePosition)
			{
				var sortIndex = -1;
				for (var j = 0; j < specSortedIntQiacubePosition.length; j++)
				{
					if (intQiacubePosition == specSortedIntQiacubePosition[j])
					{
						sortIndex = j;
					}
				}
				if (sortIndex > -1)
				{
					intNanoDropPlatePosition = sortIndex + 1;
				}
			}
			if (exreg.isRna(item))
			{
				rnaItemNo++;
				intNanoDropPlatePosition = numSpecimenItems + rnaItemNo;
			}
			
			// Calculate NanoDrop well from Qiacube position
			var row;
			var col;
			// Specimens from up to two QIAcube instruments and input RNA items placed on one NanoDrop plate
			if (intNanoDropPlatePosition > 16)
			{
				// (17 - 24) NanoDrop plate for QIAcube 'A', positions A3 - H3
				row = intNanoDropPlatePosition - 17;
				col = 3;
			}
			else if (intNanoDropPlatePosition > 8)
			{
				// ( 9 - 12) NanoDrop plate for QIAcube 'C', positions A2 - D2
				// (13 - 16) NanoDrop plate for QIAcube 'A', positions E2 - H2
				row = intNanoDropPlatePosition - 9;
				col = 2;
			}
			else if (intNanoDropPlatePosition > 0)
			{
				// ( 1 -  8) NanoDrop plate for QIAcube 'C', positions A1 - H1
				row = intNanoDropPlatePosition - 1;
				col = 1;
			}
			if (i < numItems)
			{

				if (!exreg.isDna(item))
				{
/*
					item.dna.well = Meludi.wellToAlpha(row) + (col+3);
*/
					//item.rna.well = Meludi.wellToAlpha(row) + col;
					item.rna.nanoDropWell = Meludi.wellToAlpha(row) + col;
				}
				else
				{
					//item.dna.well = null;
					item.rna.nanoDropWell = null;
				}

				Doc.element('dnaVol'+i).innerHTML = item.dna.volume;
				Doc.addOrRemoveClass('dnaVol'+i, 'nondefault', item.dna.volume != dnaDefaultVolume);
				var dnaConc = Doc.element('dnaConc'+i);
				if (dnaConc != null)
				{
					dnaConc.value = Meludi.formatNumber(item.dna.qubitConc) || '';
					dnaConc.title = '260/280=' + item.dna.nd260by280 + '\n260/230=' + item.dna.nd260by230;
					var dnaConcValue = dnaConc.value;
					if (dnaConcValue != null && dnaConcValue != '-' && dnaConcValue != '')
					{
						item.dna.qubitConc = exreg.getNumber(dnaConcValue);
					}
				}
				item.dna.quantity = item.dna.qubitConc * item.dna.volume / 1000;
				Doc.element('dnaQuantity'+i).innerHTML = Meludi.formatNumber(item.dna.quantity || null, null, 2) || '-';

				var deltaCt = Doc.element('deltaCt'+i);
				if (deltaCt != null)
				{
					deltaCt.value = Meludi.formatNumber(item.dna.deltaCt) || '';
					var deltaCtValue = deltaCt.value;
					if (deltaCtValue != null && deltaCtValue != '-' && deltaCtValue != '')
					{
						item.dna.deltaCt = exreg.getNumber(deltaCtValue);
					}
					deltaCt.innerHTML = Meludi.formatNumber(item.dna.deltaCt) || '-';
				}
			
				//Doc.element('rnaPos'+i).innerHTML = item.rna.well;
				Doc.element('rnaPos'+i).innerHTML = item.rna.nanoDropWell;
				Doc.addOrRemoveClass('rnaPos'+i, 'nondefault', item.qiacubePosition != item.originalQiacubePosition);
				
				Doc.element('rnaVol'+i).innerHTML = item.rna.volume;
				Doc.addOrRemoveClass('rnaVol'+i, 'nondefault', item.rna.volume != rnaDefaultVolume);
				var rnaConc = Doc.element('rnaConc'+i);
				if (rnaConc != null)
				{
					rnaConc.value = Meludi.formatNumber(item.rna.ndConc) || '';
					rnaConc.title = '260/280=' + item.rna.nd260by280 + '\n260/230=' + item.rna.nd260by230;
					var rnaConcValue = rnaConc.value;
					if (rnaConcValue != null && rnaConcValue != '-' && rnaConcValue != '')
					{
						item.rna.ndConc = exreg.getNumber(rnaConcValue);
					}
				}
				item.rna.quantity = item.rna.ndConc * item.rna.volume / 1000;
				Doc.element('rnaQuantity'+i).innerHTML = Meludi.formatNumber(item.rna.quantity || null, null, 2) || '-';

				var enoughQuantity = item.rna.quantity >= MINIMAL_PRE_NORMALIZE_QUANTITY;
				item.rna.flag = enoughQuantity ? null : 'NotEnoughRemainingQuantity';
				
				if (exreg.isSpecimen(item) || exreg.isRna(item))
				{
/*
					frm['rnaPreNormalize'+i].disabled = !enoughQuantity;
*/
					Doc.element('rnaQuantity'+i).innerHTML = Meludi.formatNumber(item.rna.quantity || null, null, 2) || '-';
/*
					frm['rnaPreNormalize'+i].checked = enoughQuantity;
					var warning = item.rna.quantity && !enoughQuantity;
					Wizard.setInputStatus('rnaPreNormalize'+i, warning ? 'flag' : '', warning ? 'Not enough quantity' : '');
*/
				}

				var cmnt = Doc.element('comments'+i);
				var tooltip = [];
				var comment = [];
				if (item.operatorDeliveryComment || item.comment || item.dna.comment || item.rna.comment)
				{
					// Display operator delivery comments directly in the comment field
					if (item.operatorDeliveryComment)
					{
						comment[0] = item.operatorDeliveryComment;
						tooltip[tooltip.length] = 'Op.Del.Cmt: ' + item.operatorDeliveryComment;
					}
					else
					{
						comment[0] = 'Yes';
					}
					if (item.comment) tooltip[tooltip.length] = 'Specimen: ' + item.comment;
					if (item.rna.comment) tooltip[tooltip.length] = 'RNA: ' + item.rna.comment;
					if (item.dna.comment) tooltip[tooltip.length] = 'DNA: ' + item.dna.comment;
				}
				
				var modifiedReagents = false;
				for (var rgNo = 0; rgNo < exreg.REAGENTS.length; rgNo++)
				{
					var rg = exreg.REAGENTS[rgNo];
					if (item[rg]) 
					{
						tooltip[tooltip.length] = Doc.element(rg+'.title').innerHTML + ': ' + Strings.encodeTags(item[rg]);
						modifiedReagents = true;
					}
				}
				if (modifiedReagents)
				{
					comment[comment.length] = 'Reagents';
				}
				
				if (tooltip.length > 0)
				{
					cmnt.innerHTML = comment.join(', ');
					Doc.addClass(cmnt, 'nondefault');
					cmnt.title = tooltip.join('\n');
				}
				else
				{
					cmnt.innerHTML = '';
					Doc.removeClass(cmnt, 'nondefault');
					cmnt.title = null;
				}
			}
		}
		
		if (duplicatePosition)
		{
			var duplicatePositionStr = exreg.getQiacubePosition(duplicatePosition);
			qiacubePositionsAreValid = false;
		}
		else
		{
			qiacubePositionsAreValid = true;
		}
	}

	/**
		Returns a pure integer position value from a QIAcube position
		string with optional instrument letter prefix A-Z.
		Returns `null` if input position string is `null`.

		positionStr = integer          => QIAcube position = integer (1-12)
		positionStr = Prefix + integer => QIAcube position = integer + 12*<Prefix order number 0-25>
	 */
	exreg.getIntQiacubePosition = function(positionStr)
	{
		var intPos = null;
		if (positionStr)
		{
			// Check if string contains uppercase letter + QIAcube position number 1-12
			if (/^[A-Z]([2-9]|1[0-2]?)$/.test(positionStr))
			{
				var qiaCubePrefix = positionStr.substring(0,1);
				// Get QIAcube prefix order number 0-25
				var qiacubePrefixOrder = qiacubeSortedPrefixes.indexOf(qiaCubePrefix);
				if (qiacubePrefixOrder >= 0)
				{
					intPos = parseInt(positionStr.substring(1)) + 12*qiacubePrefixOrder;
				}
			}
			if (intPos == null)
			{
				intPos = parseInt(positionStr);
			}
		}
		return intPos;
	}

	/**
		Returns a QIAcube position string with optional instrument
		letter prefix A-Z from a pure integer position value.

		Examples for qiacubeSortedPrefixes = 'CABDEFGHIJKLMNOPQRSTUVWXYZ' ("C" is default)
		intPosition (  1 -  12) => positionStr = 'C' + integer
		intPosition ( 13 -  24) => positionStr = 'A' + (integer - 12)
		intPosition (301 - 312) => positionStr = 'Z' + (integer - 300)
	 */
	exreg.getQiacubePosition = function(intPosition)
	{
		var qiacubePositionStr = null;
		if (intPosition && intPosition > 0)
		{
			var numPrefixes = qiacubeSortedPrefixes.length;
			var prefixIndex = Math.floor(intPosition/12);
			if (prefixIndex < numPrefixes)
			{
				var qiacubePrefix = qiacubeSortedPrefixes.substring(prefixIndex, prefixIndex + 1);
				var qiacubePos = intPosition - 12*prefixIndex;
				qiacubePositionStr = qiacubePrefix + qiacubePos;
			}
			else
			{
				qiacubePositionStr = '' + intPosition;
			}
		}
		return qiacubePositionStr;
	}

	exreg.editDnaRnaDetails = function(event)
	{
		var index = Data.get(event.currentTarget, 'index');
		var frm = document.forms['meludi'];
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		var item = null;
		if (index < numItems)
		{
			item = selItemsList[index];
		}
		else
		{
			// Extra QIAcube item
			item = extraItemsList[index - numItems];
		}
		var url = 'edit_dnarna_details.jsp?ID='+App.getSessionId();
		url += '&index='+index;
		url += '&mode=' + 'ExtractionReg';
		url += '&specimen=' + item.name;
		Dialogs.openPopup(url, 'EditDnaRnaDetails'+index, 750, 500);	
	}

	exreg.nanoDropFileOnChange = function()
	{
		var frm = document.forms['meludi'];
		var file = frm.nanoDropFile.files[0];
		
		var reader = new FileReader();
		reader.onload = exreg.nanoDropFileLoaded;
		reader.readAsText(file);
	}

	exreg.nanoDropFileLoaded = function(event)
	{
		exreg.parseNanoDropFile(event.target.result);
	}


	exreg.parseNanoDropFile = function(data)
	{
		var frm = document.forms['meludi'];
		nanoDropFileIsValid = false;

		var numSpecimens = 0;
		try
		{
			// Reset the existing values
			
			// Get list of selected extract source items
			var selItemsList = exreg.getSelectedItemsList();
			var numItems = selItemsList.length;
			var numNanoDropItems = 0;
			for (var i = 0; i < numItems; i++)
			{
				var item = selItemsList[i];
				if (exreg.isSpecimen(item) || exreg.isRna(item))
				{
					item.rna.ndConc = null;
					item.rna.nd260by280 = null;
					item.rna.nd260by230 = null;
					numNanoDropItems++;
				}
			}
			
			var lines = data.split(/[\n\r]+/);
			var columns = lines[0].split(/ *\t */);
			
			var sampleIndex = exreg.findColumn(columns, 'Sample ID');
			if (sampleIndex == -1) return;
			
			var wellIndex = exreg.findColumn(columns, 'Well');
			if (wellIndex == -1) return;
			
			var ndConcIndex = exreg.findColumn(columns, 'Conc.');
			if (ndConcIndex == -1) return;
			
			var nd260by280Index = exreg.findColumn(columns, '260/280');
			if (nd260by280Index == -1) return;
			
			var nd260by230Index = exreg.findColumn(columns, '260/230');
			if (nd260by230Index == -1) return;
		
			var numImported = 0;		
			for (var i = 1; i < lines.length; i++)
			{
				if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
				
				var row = lines[i].split(/ *\t */);
				if (row.length != columns.length) 
				{
					Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; Unexpected number of columns: ' + row.length);
					return;
				}

				var sampleId = row[sampleIndex];
				if (!sampleId || sampleId == 'blank') continue; // with the next line
		
				var rnaOrDna = exreg.findRnaOrDna(sampleId, i+1, 'nanoDropFile');
				if (!rnaOrDna) return;  // This is an error condition
				
				if (rnaOrDna.well != row[wellIndex])
				{
					Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; Expected to find "' + sampleId + '" in well [' + rnaOrDna.well + '] not ['  + row[wellIndex] + ']');
					return;
				}
				
				var ndConc = exreg.getNumber(row[ndConcIndex]);
				if (isNaN(ndConc))
				{
					Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; No valid NDConc value for "' + sampleId + '": ' + row[ndConcIndex]);
					return;
				}
				
				// If there are multiple entries for the same item, keep the largest NDConc
				var isFirstValue = rnaOrDna.ndConc == null;
				if (isFirstValue || ndConc > rnaOrDna.ndConc)
				{
					var nd260by280 = exreg.getNumber(row[nd260by280Index]);
					if (isNaN(nd260by280))
					{
						Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; No valid 260/280 value for "' + sampleId + '": ' + row[nd260by280Index]);
						return;
					}
					
					var nd260by230 = exreg.getNumber(row[nd260by230Index]);
					if (isNaN(nd260by230))
					{
						Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; No valid 260/230 value for "' + sampleId + '": '+row[nd260by230Index]);
						return;
					}
					
					// Store values
					rnaOrDna.ndConc = ndConc;
					rnaOrDna.nd260by280 = nd260by280;
					rnaOrDna.nd260by230 = nd260by230;

					// Only update counter for the first value (not when replacing a lower value)
					if (isFirstValue)
					{
						numImported++;
					}
				}
				
			}

			if (numImported != numNanoDropItems)
			{
				Wizard.setInputStatus('nanoDropFile', 'invalid', 'Could not find NanoDrop values for all ' + numNanoDropItems + ' samples: ' + numImported);
				return;
			}

			Wizard.setInputStatus('nanoDropFile', 'valid');
			nanoDropFileIsValid = true;
		}
		finally
		{
			exreg.updateDetailsTable();
		}
	}

	exreg.qubitFileOnChange = function()
	{
		var frm = document.forms['meludi'];
		var file = frm.qubitFile.files[0];
		
		var reader = new FileReader();
		reader.onload = exreg.qubitFileLoaded;
		reader.readAsText(file);
	}

	exreg.qubitFileDilutionFactorOnChange = function()
	{
		var frm = document.forms['meludi'];
		var dilutionFactor = frm.qubitFileDilutionFactor.value;
		dilutionFactor = exreg.getNumber(dilutionFactor);		

		if (isNaN(dilutionFactor))
		{
			Wizard.setInputStatus('qubitFileDilutionFactor', 'invalid', 'Dilution factor must be a number.');
			return;
		}
		else if (dilutionFactor <= 0)
		{
			Wizard.setInputStatus('qubitFileDilutionFactor', 'invalid', 'Dilution factor must be >= 0.');
			return;
		}
		frm.qubitFileDilutionFactor.value = dilutionFactor;
		Wizard.setInputStatus('qubitFileDilutionFactor', 'valid');
	}

	exreg.qubitFileLoaded = function(event)
	{
		exreg.parseQubitFile(event.target.result);
	}

	exreg.parseQubitFile = function(data)
	{
		var frm = document.forms['meludi'];
		var dilutionFactorDefault = frm.qubitFileDilutionFactor.value;
		qubitFileIsValid = false;

		var numSpecimens = 0;
		try
		{
			// Reset the existing values
			
			// Get list of selected extract source items
			var selItemsList = exreg.getSelectedItemsList();
			var numItems = selItemsList.length;
			var qubitItemsList = [];
			var numQubitItems = 0;
			var qubitFileInfoText = null;
			// Reset existing values, if not explicitly entered in input field
			for (var i = 0; i < numItems; i++)
			{
				var item = selItemsList[i];
				if (exreg.isSpecimen(item) || exreg.isDna(item))
				{
					var dnaConc = frm['dnaConc'+i];
					if (dnaConc != null)
					{
						var dnaConcValue = frm['dnaConc'+i].value;
						if (dnaConcValue != null && dnaConcValue != '' && dnaConcValue != '-')
						{
							item.dna.qubitConc = exreg.getNumber(dnaConcValue);
						}
						else
						{
							item.dna.qubitConc = null;
							item.dna.nd260by280 = null;
							item.dna.nd260by230 = null;
						}
					}
					else
					{
						item.dna.qubitConc = null;
						item.dna.nd260by280 = null;
						item.dna.nd260by230 = null;
					}
					qubitItemsList[numQubitItems] = item;
					numQubitItems++;
				}
			}
			
			var lines = data.split(/[\n\r]+/);
			//var columns = lines[0].split(/ *\t */);
			var columns = lines[0].split(/ *, */);
			
			var nameIndex = exreg.findColumn(columns, 'Name');
			if (nameIndex == -1) return;
			
			var assayConcIndex = exreg.findColumn(columns, 'Assay Conc.');
			if (assayConcIndex == -1) return;
			
			var unitsIndex = exreg.findColumn(columns, 'Units');
			if (unitsIndex == -1) return;

			var dilutionFactorIndex = exreg.findColumn(columns, 'Dilution Factor');
			if (dilutionFactorIndex == -1) return;
			
			// Entry lines come in reversed chronological order
			var qubitConcReversedList = [];
			var numConcValues = 0;
			for (var i = 1; i < lines.length; i++)
			{
				if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
				
				//var row = lines[i].split(/ *\t */);
				var row = lines[i].split(/ *, */);
				if (row.length != columns.length) 
				{
					Wizard.setInputStatus('qubitFile', 'invalid', 'On line ' + (i+1) + '; Unexpected number of columns: ' + row.length);
					return;
				}
				
				var sampleName = row[nameIndex];
				if (!sampleName) continue; // with the next line if sample name missing
		
				var assayConc = row[assayConcIndex];
				var units = row[unitsIndex];
				var dilutionFactor = row[dilutionFactorIndex];
				//
				// Note: Up to this point, input variables are of type string.
				// If the value represents a number, the variable needs to be
				// converted to type number, which is important e.g. when calling
				// function toFixed(numDecimals), which is defined for numbers, but not for strings.
				//
				if (assayConc != null && !isNaN(assayConc))
				{
					assayConc = exreg.getNumber(assayConc);
				}

				if (dilutionFactor == null || dilutionFactor == '')
				{
					dilutionFactor = dilutionFactorDefault;
				}

				if (!isNaN(assayConc))
				{
					// Multiply with dilution factor
					assayConc *= dilutionFactor;
				}
				if (assayConc && units && units == 'ng/ml')
				{
					if (!isNaN(assayConc))
					{
						// Convert to ng/microliter = microgram/milliliter
						assayConc /= 1000.0;
					}
				}

				if (assayConc)
				{
					qubitConcReversedList[numConcValues] = assayConc;
					numConcValues++;
				}			
			}
					
			// Reverse list to get values in correct order
			var qubitConcList = [];
			for (var i = 0; i < numConcValues; i++)
			{
				qubitConcList[i] = qubitConcReversedList[(numConcValues - 1) - i];
			}

			var numImported = 0;
			var qubitConcIndex = 0;	
			for (var i = 0; i < numItems; i++)
			{
				var item = selItemsList[i];
				if (exreg.isSpecimen(item) || exreg.isDna(item))
				{
					var qubitConc = qubitConcList[qubitConcIndex];
					if (isNaN(qubitConc))
					{
						// This case is often "<0.50" (assay concentration < 0.50 ng/ml excluding dilution)
						var qubitItem = qubitItemsList[qubitConcIndex];
/*
						Wizard.setInputStatus('qubitFile', 'invalid', 'On line ' + (i+1) + ': No valid Assay Conc value for "' + qubitItem.name + '": ' + qubitConc);
						return;
*/
						qubitFileInfoText = 'Assay conc. value for "' + qubitItem.name + '": ' + qubitConc + ' set to 0.0.';
						qubitConc = 0.0;
					}
					qubitConcIndex++;
					numImported++;

					var dnaConc = frm['dnaConc'+i];
					if (dnaConc != null)
					{
						var dnaConcValue = frm['dnaConc'+i].value;
						if (dnaConcValue != null && dnaConcValue != '' && dnaConcValue != '-')
						{
							item.dna.qubitConc = exreg.getNumber(dnaConcValue);
						}
						else
						{
							item.dna.qubitConc = qubitConc;
						}
					}
					else
					{
						item.dna.qubitConc = qubitConc;
					}
				}
			}

			if (numImported != numQubitItems)
			{
				Wizard.setInputStatus('qubitFile', 'invalid', 'Could not find Qubit values for all ' + numQubitItems + ' samples: ' + numImported);
				return;
			}

			if (qubitFileInfoText)
			{
				Wizard.setInputStatus('qubitFile', 'flag', qubitFileInfoText);
			}
			else
			{
				Wizard.setInputStatus('qubitFile', 'valid');
			}
			qubitFileIsValid = true;
		}
		finally
		{
			exreg.updateDetailsTable();
		}
	}

	exreg.parseDnaInput = function(data)
	{
		var frm = document.forms['meludi'];
		qubitFileIsValid = false;

		var numQubitItems = 0;
		try
		{
			// Get list of selected extract source items
			var selItemsList = exreg.getSelectedItemsList();
			var numItems = selItemsList.length;

			// Reset the existing values
			for (var i = 0; i < numItems; i++)
			{
				var item = selItemsList[i];
				var dnaConc = frm['dnaConc'+i];
				if (dnaConc != null)
				{
					var dnaConcValue = frm['dnaConc'+i].value;
					if (dnaConcValue != null && dnaConcValue != '' && dnaConcValue != '-')
					{
						item.dna.qubitConc = exreg.getNumber(dnaConcValue);
					}
					else
					{
						item.dna.qubitConc = null;
					}
					numQubitItems++;
				}
				else
				{
					item.dna.qubitConc = null;
				}
			}
			
			var lines = data.split(/[\n\r]+/);
			var columns = lines[0].split(/ *\t */);
			
			var sampleIndex = exreg.findColumn(columns, 'Sample ID');
			if (sampleIndex == -1) return;
			
			var wellIndex = exreg.findColumn(columns, 'Well');
			if (wellIndex == -1) return;
			
			var qubitConcIndex = exreg.findColumn(columns, 'Conc.');
			if (qubitConcIndex == -1) return;
			
			var numImported = 0;		
			for (var i = 1; i < lines.length; i++)
			{
				if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
				
				var row = lines[i].split(/ *\t */);
				if (row.length != columns.length) 
				{
					Wizard.setInputStatus('qubitFile', 'invalid', 'On line ' + (i+1) + '; Unexpected number of columns: ' + row.length);
					return;
				}
				
				var sampleId = row[sampleIndex];
				if (!sampleId || sampleId == 'blank') continue; // with the next line
		
				var rnaOrDna = exreg.findRnaOrDna(sampleId, i+1, 'qubitFile');
				if (!rnaOrDna) return;  // This is an error condition

				// No check for correct well since DNA is not stored on plate

				var qubitConc = exreg.getNumber(row[qubitConcIndex]);
				if (isNaN(qubitConc))
				{
					Wizard.setInputStatus('qubitFile', 'invalid', 'On line ' + (i+1) + '; No valid qubitConc value for "' + sampleId + '": ' + row[qubitConcIndex]);
					return;
				}
				
				// If there are multiple entries for the same item, keep the largest QubitConc
				var isFirstValue = rnaOrDna.qubitConc == null;
				if (isFirstValue || qubitConc > rnaOrDna.qubitConc)
				{
					// Store values
					if (rnaOrDna.qubitConc == null || rnaOrDna.qubitConc == '')
					{
						rnaOrDna.qubitConc = qubitConc;
						isFirstValue = false;
					}

					// Only update counter for the first value (not when replacing a lower value)
					if (isFirstValue)
					{
						numImported++;
					}
				}
				
			}

/*
			if (numImported != numSpecimens)
			{
				Wizard.setInputStatus('qubitFile', 'invalid', 'Could not find values for all ' + numSpecimens + ' samples: ' + numImported);
				return;
			}
*/
/*
			Wizard.setInputStatus('qubitFile', 'valid');
			qubitFileIsValid = true;
*/
		}
		finally
		{
			exreg.updateDetailsTable();
		}
	}

	exreg.findColumn = function(columns, col)
	{
		var colIndex = columns.indexOf(col);
		if (colIndex == -1)
		{
			Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line 1; Can\'t find "' + col + '" column');
		}
		return colIndex;
	}

	exreg.qPcrFileOnChange = function()
	{
		var frm = document.forms['meludi'];
		var file = frm.qPcrFile.files[0];
		
		var reader = new FileReader();
		reader.onload = exreg.qPcrFileLoaded;
		reader.readAsText(file);
	}

	exreg.qPcrFileLoaded = function(event)
	{
		exreg.parseQPcrFile(event.target.result);
	}

	exreg.parseQPcrFile = function(data)
	{
		var frm = document.forms['meludi'];
		qPcrFileIsValid = false;
		var qPcrFileInfoText = null;

		var numQPcrItems = 0;
		try
		{
			// Reset the existing values
			
			// Get list of selected extract source items
			var selItemsList = exreg.getSelectedItemsList();
			var numItems = selItemsList.length;
			for (var i = 0; i < numItems; i++)
			{
				var item = selItemsList[i];
				if (exreg.isSpecimen(item) || exreg.isDna(item))
				{
					item.dna.deltaCt = null;
					numQPcrItems++;
				}
			}

			// Find column separator as first character after first occurrence of search string
			var separator = exreg.findSeparator(data, "Well");
			if (separator == null)
			{
				// Comma is default separator
				separator = ',';
			}
			var columnSplitRegexp = new RegExp(" *" + separator + " *",'');
			var lines = data.split(/[\n\r]+/);
			//var columns = lines[0].split(/ *\t */);
			//var columns = lines[0].split(/ *, */);
			//var columns = lines[0].split(/ *; */);
			var columns = lines[0].split(columnSplitRegexp);
			
			var sampleIndex = exreg.findQPcrColumn(columns, 'Sample');
			if (sampleIndex == -1) return;
			var deltaCtIndex = exreg.findQPcrColumn(columns, 'Delta Ct');
			if (deltaCtIndex == -1) return;

			var numImported = 0;		
			for (var i = 1; i < lines.length; i++)
			{
				if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
				//var row = lines[i].split(/ *\t */);
				//var row = lines[i].split(/ *, */);
				// Exchange decimal comma for decimal dot inside double quotes
				var procLine = exreg.setDecimalDotInsideDoubleQuotes(lines[i]);
				//var row = procLine.split(/ *, */);
				//var row = procLine.split(/ *; */);
				var row = procLine.split(columnSplitRegexp);
				if (row.length != columns.length) 
				{
					Wizard.setInputStatus('qPcrFile', 'invalid', 'On line ' + (i+1) + '; Unexpected number of columns: ' + row.length + " instead of " + columns.length);
					return;
				}
				
				var sampleId = row[sampleIndex];
				if (!sampleId || sampleId == 'blank' || sampleId == '' || sampleId == 'QCT') continue; // with the next line
		
				//var messageId = 'qPcrFile';
				var messageId = null;
				var rnaOrDna = exreg.findRnaOrDna(sampleId, i+1, messageId);
				if (!rnaOrDna)
				{
					// No RNA/DNA found with sample ID
					qPcrFileInfoText = 'On line ' + (i+1) + '; Can\'t find RNA/DNA with name "' + sampleId + '"';
					// Continue with the next line
					continue;
				}
/*				
				if (rnaOrDna.well != row[wellIndex])
				{
					Wizard.setInputStatus('qPcrFile', 'invalid', 'On line ' + (i+1) + '; Expected to find "' + sampleId + '" in well [' + rnaOrDna.well + '] not ['  + row[wellIndex] + ']');
					return;
				}
*/			
				var isFirstValue = rnaOrDna.deltaCt == null;
				if (isFirstValue)
				{
					//var deltaCt = exreg.getNumber(row[deltaCtIndex]);
					var deltaCtStr = row[deltaCtIndex];
					// Remove any double quotes surrounding value string
					if (deltaCtStr.indexOf('"') == 0)
					{
						deltaCtStr = deltaCtStr.substring(1, deltaCtStr.length - 1);
					}
					var deltaCt = exreg.getNumber(deltaCtStr);
					if (isNaN(deltaCt))
					{
						Wizard.setInputStatus('qPcrFile', 'invalid', 'On line ' + (i+1) + '; No valid delta-Ct value for "' + sampleId + '": ' + row[deltaCtIndex]);
						return;
					}
					else
					{
						// Store values
						rnaOrDna.deltaCt = deltaCt;
						numImported++;
					}
				}
			}

			if (numImported != numQPcrItems)
			{
				Wizard.setInputStatus('qPcrFile', 'invalid', 'Could not find values for all ' + numQPcrItems + ' samples: ' + numImported);
				return;
			}

			if (qPcrFileInfoText)
			{
				Wizard.setInputStatus('qPcrFile', 'flag', qPcrFileInfoText);
			}
			else
			{
				Wizard.setInputStatus('qPcrFile', 'valid');
			}
			qPcrFileIsValid = true;
		}
		finally
		{
			exreg.updateDetailsTable();
		}
	}

	exreg.findSeparator = function(data, searchString)
	{
		// Searches for the first line containing the input search string, then returns next character as separator
		var separator = null;
		if (data != null && searchString != null)
		{
			var lines = data.split(/[\n\r]+/);
			var found = false;
			for (var i = 0; (i < lines.length && !found); i++)
			{
				if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
				var line = lines[i];
				var index = line.indexOf(searchString);
				if (index >= 0)
				{
					// Get next character after first column header as separator
					var separatorIndex = index + searchString.length;
					separator = line.substring(separatorIndex, separatorIndex+1);
					found = true;
				}
			}
		}
		return separator;
	}

	exreg.setDecimalDotInsideDoubleQuotes = function(line)
	{
		var lineParts = line.split(/ *" */);
		var procLine = lineParts[0];
		var numAddedDblQts = 0;
		for (var i = 1; i < lineParts.length; i++)
		{
			var str = lineParts[i];
			if (i % 2 == 1)
			{
				// Inside double quotes
				// Replace decimal comma with decimal dot
				str = str.replace(/,/, '.');
			}
			// Add string to processed line
			procLine += '"' + str;
			numAddedDblQts++;
		}
		if (numAddedDblQts % 2 == 1)
		{
			// Add terminal double quote			
			procLine += '"';
		}
		// Return processed line
		return procLine;
	}

	exreg.findQPcrColumn = function(columns, col)
	{
		var colIndex = columns.indexOf(col);
		if (colIndex == -1)
		{
			Wizard.setInputStatus('qPcrFile', 'invalid', 'On line 1; Can\'t find "' + col + '" column');
		}
		return colIndex;
	}

	exreg.getNumber = function(sValue)
	{
		return parseFloat(sValue.replace(/,/, '.'));
	}

	exreg.findRnaOrDna = function(sampleId, lineNo, messageId)
	{
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			var test_rna_name = item.rna.name;
			var test_dna_name = item.dna.name;
			if (exreg.isRna(item))
			{
				test_rna_name = item.name;
			}
			if (exreg.isDna(item))
			{
				test_dna_name = item.name;
			}
			if (test_rna_name == sampleId)
			{
				return item.rna;
			}
			else if (test_dna_name == sampleId)
			{
				return item.dna;
			}
		}
		if (messageId)
		{
			Wizard.setInputStatus(messageId, 'invalid', 'On line ' + lineNo + '; Can\'t find RNA/DNA with name "' + sampleId + '"');
		}
		return null;
	}

	exreg.generateFakeNanoDropFile = function()
	{
		if (location.protocol == 'https:')
		{
			Forms.showNotification('btnFakeNanoDropFile', 'For security reasons, this function can\'t be used on HTTPS servers.');
			return;
		}
		
		var FILE_HEADER = ['Well', 'Sample ID', 'Conc.', '260/280', '260/230'];
		var data = FILE_HEADER.join('\t')+'\n';
		
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exreg.isSpecimen(item) || exreg.isRna(item))
			{
				var line = [];
				// RNA
				line[0] = item.rna.well;
				if (exreg.isSpecimen(item))
				{
					line[1] = item.rna.name;
				}
				else if (exreg.isRna(item))
				{
					line[1] = item.name;
				}
				line[2] = Numbers.formatNumber(Math.random()*100+10, 2);
				line[3] = Numbers.formatNumber(Math.random()*.5+2, 2);
				line[4] = Numbers.formatNumber(Math.random()+1, 2);
				data += line.join('\t') + '\n';			
			}
		}
		exreg.parseNanoDropFile(data);
	}

	exreg.generateFakeDnaInput = function()
	{
		if (location.protocol == 'https:')
		{
			Forms.showNotification('btnFakeDnaInput', 'For security reasons, this function can\'t be used on HTTPS servers.');
			return;
		}
		
		var FILE_HEADER = ['Well', 'Sample ID', 'Conc.', '260/280', '260/230'];
		var data = FILE_HEADER.join('\t')+'\n';

		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exreg.isSpecimen(item) || exreg.isDna(item))
			{
				var line = [];
				// DNA
				line[0] = item.dna.well;
				if (exreg.isSpecimen(item))
				{
					line[1] = item.dna.name;
				}
				else if (exreg.isDna(item))
				{
					line[1] = item.name;
				}
				line[2] = Numbers.formatNumber(Math.random()*100+10, 2);
				line[3] = Numbers.formatNumber(Math.random()*.5+2, 2);
				line[4] = Numbers.formatNumber(Math.random()+1, 2);
				data += line.join('\t') + '\n';			
			}
		}
		exreg.parseDnaInput(data);
	}

	exreg.generateFakeQPcrFile = function()
	{
		if (location.protocol == 'https:')
		{
			Forms.showNotification('btnFakeQPcrFile', 'For security reasons, this function can\'t be used on HTTPS servers.');
			return;
		}
		var FILE_HEADER = ['Well', 'Fluor', 'Content', 'Sample', 'Cq', 'Ct', 'Delta Ct'];
		var data = FILE_HEADER.join(',')+'\n';
		
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
			if (exreg.isSpecimen(item) || exreg.isDna(item))
			{
				for (var j = 0; j < 3; j++)
				{
					var line = [];
					// DNA
					line[0] = item.dna.well;
					line[1] = 'SYBR';
					line[2] = 'Unkn';
					if (exreg.isSpecimen(item))
					{
						line[3] = item.dna.name;
					}
					else if (exreg.isDna(item))
					{
						line[3] = item.name;
					}
					if (j == 0)
					{
						line[4] = '"' + Numbers.formatNumber(Math.random()*100+10, 2) + '"';
						line[5] = '"' + Numbers.formatNumber(Math.random()*.5+2, 2) + '"';
						line[6] = '"' + Numbers.formatNumber(Math.random()+1, 2) + '"';
					}
					else
					{
						line[4] = '"' + Numbers.formatNumber(Math.random()*100+10, 2) + '"';
						line[5] = '';
						line[6] = '';
					}
					data += line.join(',') + '\n';
				}	
			}
		}
		exreg.parseQPcrFile(data);
	}

	exreg.validateStep3 = function(event)
	{
		var valid = true;
/*
		valid &= nanoDropFileIsValid;
*/
		valid &= qiacubePositionsAreValid;
		
		if (!valid) event.preventDefault();
	}


	exreg.submit = function()
	{
		var frm = document.forms['meludi'];
		
		var submitInfo = {};
		submitInfo.items = [];

		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		submitInfo.numExtraQiacubeItems = numExtraQiacubeItems;
		for (var i = 0; i < numItems; i++)
		{
			var item = selItemsList[i];
/*
			if (exreg.isSpecimen(item) || exreg.isRna(item))
			{
				item.rna.preNormalize = frm['rnaPreNormalize'+i].checked;
			}
			else
			{
				item.rna.preNormalize = false;
			}
*/
			submitInfo.items[submitInfo.items.length] = item;
		}
		
		// Extraction
		submitInfo.startPlateName = frm.startPlateName.value;
		submitInfo.startPlateId = frm.startPlateId.value;
		submitInfo.isolationDate = frm.isolationDate.value;
		submitInfo.extractionProtocol = parseInt(frm.extractionProtocolId.value, 10);
		submitInfo.rnaDefaultVolume = exreg.getNumber(frm.rnaDefaultVolume.value);
		submitInfo.dnaDefaultVolume = exreg.getNumber(frm.dnaDefaultVolume.value);
		
		// QIAcube
		submitInfo.qiacubeDate = frm.qiacubeDate.value;
		submitInfo.qiacubeRunNo = parseInt(frm.qiacubeRunNo.value, 10);
		submitInfo.qiacubeOperator = frm.qiacubeOperator.value;

		// Reagents
		for (var i = 0; i < exreg.REAGENTS.length; i++)
		{
			var rg = exreg.REAGENTS[i];
			submitInfo[rg] = frm[rg].value;
		}

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterDnaRna';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, exreg.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	exreg.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
		Doc.show('gocreate');
		trackingSheetPreview = false;
		if (response.createdRna && response.createdRna.length > 0)
		{
			createdRna = response.createdRna;
/*
			Doc.show('rnaQcWizard');
*/
		}
	}

	exreg.createProtocol = function()
	{
		var frm = document.forms['meludi'];
		var submitInfo = {};
		submitInfo.items = [];

		submitInfo.docMode = 'report';
		// Get list of selected extract source items
		var selItemsList = exreg.getSelectedItemsList();
		var numItems = selItemsList.length;
		// Get number of extra QIAcube items
		var numExtraQiacubeItems = 0;
		var numExtraQiacubeItemsStr = frm.numExtraQiacubeItems.value;
		if (numExtraQiacubeItemsStr && (numExtraQiacubeItemsStr != ''))
		{
			numExtraQiacubeItems = parseInt(numExtraQiacubeItemsStr, 10);
		}
		submitInfo.numExtraQiacubeItems = numExtraQiacubeItems;
		var totNumItems = numItems + numExtraQiacubeItems;
		for (var i = 0; i < totNumItems; i++)
		{
			var item = null;
			if (i < numItems)
			{
				item = selItemsList[i];
			}
			else
			{
				// Extra QIAcube item
				item = extraItemsList[i - numItems];
			}
/*
			if (exreg.isSpecimen(item) || exreg.isRna(item))
			{
				item.rna.preNormalize = frm['rnaPreNormalize'+i].checked;
			}
			else
			{
				item.rna.preNormalize = false;
			}
*/
			submitInfo.items[submitInfo.items.length] = item;
		}

		// Extraction
		submitInfo.startPlateName = frm.startPlateName.value;
		submitInfo.numExtraQiacubeItems = frm.numExtraQiacubeItems.value;
		submitInfo.isolationDate = frm.isolationDate.value;
		submitInfo.extractionProtocol = parseInt(frm.extractionProtocolId.value, 10);
		var extractionProtocolName = frm.extractionProtocolName.value;
		var protocolVersionNo = '';
		if (extractionProtocolName)
		{
			protocolVersionNo = extractionProtocolName;
			// Remove part of string up to and including last "v"
			var vIndex = protocolVersionNo.indexOf('v');
			while (vIndex >= 0)
			{
				// Remove part of string up to and including first "v"
				protocolVersionNo = protocolVersionNo.substring(vIndex + 1);
				vIndex = protocolVersionNo.indexOf('v');
			}
		}
		var protocolPreviewFlag = null;
		if (trackingSheetPreview)
		{
			protocolPreviewFlag = 'true';
		}
		submitInfo.extractionProtocolName = extractionProtocolName;
		submitInfo.protocolVersionNo = protocolVersionNo;
		submitInfo.protocolPreviewFlag = protocolPreviewFlag;
		submitInfo.dnaDefaultVolume = exreg.getNumber(frm.dnaDefaultVolume.value);
		
		// QIAcube
		submitInfo.qiacubeDate = frm.qiacubeDate.value;
		submitInfo.qiacubeRunNo = parseInt(frm.qiacubeRunNo.value, 10);
		submitInfo.qiacubeOperator = frm.qiacubeOperator.value;
		submitInfo.allPrepFfpeKit = frm.allPrepFfpeKit.value;

		// Save values in hidden input fields for retrieval after HTML request
		var submitInfoJsonStr = JSON.stringify(submitInfo);
		frm.hiddenSubmitInfo.value = submitInfoJsonStr;
		frm.hiddenNumItems.value = submitInfo.items.length;
		frm.hiddenCounterBalanceInfoRow.value = counterBalanceInfoRow;

		// Calling frm.submit() will change jsp file to extraction_protocol2.jsp
		frm.submit();
	}

/*
	exreg.goRnaQc = function()
	{
		var url = 'rnaqc_aliquot.jsp?ID='+App.getSessionId();
		for (var i = 0; i < createdRna.length; i++)
		{
			url += '&rna='+createdRna[i];
		}
		location.href = url;
	}
*/
	
	return exreg;
}();

Doc.onLoad(ExtractionReg.initPage);

